function disp(obj)
% DISP displays information of an NCS object when the NCS object name is
% typed in the command window
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------

    disp('NCS object')
    disp(' ')

    disp('PLANT')
    disp(['   state dimension:  ' num2str(size(obj.A,1))])
    disp(['   input dimension:  ' num2str(size(obj.B,2))])
    disp(['   output dimension: ' num2str(size(obj.C,1))])
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    disp('CONTROLLER')
    disp(['   type:             ' obj.ctrlType])
       
    if strcmp(obj.ctrlType,'Static Feedback')    
        if length(obj.K)==1
            disp('   node dependent:   No')
        else
            disp('   node dependent:   Yes')
        end
    else
        if length(obj.Ac)==1
            disp('   node dependent:   No')
        else
            disp('   node dependent:   Yes')
        end
    end
    if strcmp(obj.ctrlType,'Static Feedback')
        disp('   state dimension:  0')
    else
        disp(['   state dimension:  ' num2str(size(obj.Ac{1},1))])
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    disp('NETWORK')
    
    if abs(obj.h(1)-obj.h(2))<1e-12 && abs(obj.tau(1)-obj.tau(2))<1e-12
        disp('   Timing Jitter:    NO')
    else
        disp('   Timing Jitter:    YES')
        disp(['      trans intvls:  [' num2str(obj.h(1)) ',' num2str(obj.h(2)) ']'])
        disp(['      delays:        [' num2str(obj.tau(1)) ',' num2str(obj.tau(2)) ']'])
    end
    
    disp(['   max dropouts:     ' num2str(obj.delta)])
    if length(obj.gammaU)==1
        disp('   sharing:          NO')
    else
        disp('   sharing:          YES')
        disp(['      nodes:         ' num2str(length(obj.gammaU))])
        disp(['      protocol:      ' obj.protocol]) 
    end 
    
end