function A = evaluateDtNcsModel(ncs,h,tau,sigma)
%EVALUATEDTNCSMODEL evaluates a NCS at a constant transmission interval,
%                   constant delay and certain node, returning a matrix 
%                   which desribes the closed-loop dynamics.
%
%   A = EVALUATEDTNCSMODEL(NCS,TXINTVL,DELAY,NODE) evaluates the NCS model
%   at a constant transmission interval, TXINTVL, constant delay, DELAY, and
%   certain node, NODE. This function returns a matrix which desribe the
%   closed-loop dynamics
%
%           x_{k+1} = A x_{k}
%
%   where x_{k} consists of the plant states, controller states,
%   network-induced error y and network-induced error u.
%
%   INPUTS
%   ------------------------------------------------------
%      NCS      - NCS object created with the ncsEditor
%      TXINTVL  - transmission interval value
%      DELAY    - delay value
%      NODE     - node value
%
%   OUTPUTS
%   ------------------------------------------------------
%      A - closed-loop system matrix
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------
    
    dtNcsMdl = genDtNcsModel(ncs,0);
    dtNcsMdl.tPoly = [h tau];
    model = genPolyOvrAprx(dtNcsMdl,'GNB',1,1,0,0);
    if size(model.Ab,2)>1
        error('something went horribly wrong')
    end
    A=model.Ab{sigma};
    
end