function dtNcsModel = getDtNcsModel(obj)
% GETDTMODELMATRICES From the properties of an NCS class object, generates the 
%                  matrices A{l},B{l},C{l},D{l},E{l},J{l} and Atilde{l}, l=1,...,N,  
%                  which describe the following discrete-time model:
%
%                  x_{k+1} = (A{l} + B{l}* Phi_{k,l} *C{l})x_{k} +
%                              (E{l} + D{l}* Phi_{k,l} *J{l})w_{k}
%
%                  where x_k is the value of the NCS state vector at time k, w_k 
%                  is a disturbance and the network uncertainty has the form
%                  Phi_{k,l}=blkdiag(A_{hk,l},E_{hk,l},E_{hk-tauk,l}).  Atilde{l} 
%                  is used for defining the matrices Phi_{k,l}. See 
%                  [Bauer et. al. NECSYS, 2012]  eqn (1) for more info.
%
%   DTNCSMDL = GETDTMODELMATRICES(NCS)
%   Extracts the information in an NCS object and generates the matrices 
%   A{l},B{l},C{l},D{l},E{l},J{l} and stores them in DTNCSMDL.  
%
%   OUTPUT
%   ------------------------------------------------------
%       DTNCSMDL    - strucuture containing information about 
%                     the discrete-time NCS model
%                     ~~~~Fields~~~~
%                     DTNCSMDL.A       - A matrix assoc. with model 
%                     DTNCSMDL.B       - B matrix assoc. with model
%                     DTNCSMDL.C       - C matrix assoc. with model
%                     DTNCSMDL.D       - D matrix assoc. with model
%                     DTNCSMDL.E       - E matrix assoc. with model
%                     DTNCSMDL.J       - J matrix assoc. with model
%                     DTNCSMDL.Atilde  - matrix assoc. with uncertainty
%                     ~~~~~~~~~~~~~~~~~~
%
%   Customization Tip:
%     *The structure which is output, DTNCSMDL, can be used in custom
%      overaproximation techniques.
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------
    
    dtNcsModel = genDtNcsModel(obj);
        
end

