function network = getNetwork(obj)
% GETNETWORK Returns the network parameters of an NCS object. 
%
% NETWORK = GETNETWORK(NCS) returns a structure containing the network parameters 
%
%   txIntvls   = [txIntvl_min txIntvl_max]
%   delays     = [delay_min delay_max]
%   tPoly      = list of vertices which define polygon that refines the uncertain region
%   timingDist = list of [txIntl, delay, probability(in %)] entries used for stochastic analysis
%   maxDrops   = integer indicating the max drops possible
%   gammaU     = set of matrices indicating which plant inputs are associated
%                with which node number
%   gammaY     = set of matrices indicating which plant outputs are associated
%                with which node number
%   protocol   = 'RR' for Round Robin or 'TOD' for Try-Once-Discard protocol
%   quanitzer  = 'none', 'uniform', 'log' specifying type of quanizer used
%
% See also SETNETWORK
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------
    network.txIntvls    = [obj.h(1) obj.h(2)];
    network.delays      = [obj.tau(1) obj.tau(2)];
    network.tPoly       = obj.tPoly;
    network.timingDist  = obj.timingDist;
    network.maxDrops    = obj.delta(1);
    network.nodesU      = obj.gammaU;
    network.nodesY      = obj.gammaY;
    network.protocol	= obj.protocol;
    network.quantizer   = obj.quantizer;
    
end