function plotTimingUncertaintyRegion(obj)
% PLOTTIMINGUNCERTAINTYREGION plots the uncertainty region associated with an 
%                             NCS object.
%
% PLOTTIMINGUNCERTAINTYREGION(NCS) plots both the uncertainty region defined
% by the txIntvl and delay bounds as well as the refined region (if defined).
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------    
    hmin    = obj.getNetwork.txIntvls(1);
    hmax    = obj.getNetwork.txIntvls(2);
    taumin  = obj.getNetwork.delays(1);
    taumax  = obj.getNetwork.delays(2);
    

    verts=genTimingUncertaintyRegion(hmin,hmax,taumin,taumax);
    m=max(max(verts));
    
    figure;
    if size(verts,1)==1
        plot(verts(:,1),verts(:,2),'r.','MarkerSize',4);
    else
        patch(verts(:,1),verts(:,2),'r','EdgeColor','r');
    end
    hold all;
    if ~isempty(obj.tPoly)
        Rverts=obj.tPoly;
        patch(Rverts(:,1),Rverts(:,2),'b','EdgeColor','b');
        plot([0 1.2*m],[0 1.2*m],'k--');
        legend('non-refined region','refined region','small-delay boundry');
    else
        plot([0 1.2*m],[0 1.2*m],'k--');
        legend('non-refined region','small-delay boundry');
    end
    
    axis([0 1.2*m 0 1.2*m]);
    ylabel('delays (s)');
    xlabel('transmission intervals (s)');
    title('Network-Timing Uncertainty Region');
end