function obj = setPlant(obj,arg1,arg2,arg3)
% SETPLANT Sets the plant of an NCS object. 
%
% NCS = SETPLANT(NCS,A,B,C) sets the plant of the NCS object such that the
% dynamics are
%
%               \dot{x}= A*x + B*u
%                    y = C*x
%
% The dimensions of B and C must correspond to the number of inputs and
% outputs of the NCS object.
%
% NCS = SETPLANT(NCS,PLANT) can also be used, where PLANT is the 
% structure resulting from the method GETPLANT.
%
%   See also GETPLANT
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------

    if nargin == 2
        %check if all fields exist
        if ~isfield(arg1,'A')
            error('A must be a field in the plant object')
        elseif ~isfield(arg1,'B')
            error('B must be a field in the plant object')
        elseif ~isfield(arg1,'C')
            error('C must be a field in the plant object')
        end

        A = arg1.A;
        B = arg1.B;
        C = arg1.C;
    elseif nargin == 4
        A = arg1;
        B = arg2;
        C = arg3;
    else
        error('wrong input.')
    end



     if isnumeric(A) == 0
        error('A must be a matrix of type double')
     elseif size(A,1) ~= size(A,2)
        error('A matrix must be square')
     end

     if isnumeric(B) == 0
        error('B must be a matrix of type double') 
     elseif size(B,1) ~= size(A,1)
         error('The number of rows of B must be equal to A')
     elseif size(B,2) ~= obj.nu
         error('The number of plant inputs is not equal to the number of controller outputs')
     end

     if isnumeric(C) == 0
        error('C must be a matrix of type double') 
     elseif size(C,2) ~= size(A,1)
        error('The number of columns of C must be equal to A')
    elseif size(C,1) ~= obj.ny
         error('The number of plant inputs is not equal to the number of controller outputs')
     end

    obj.A = A;
    obj.B = B;
    obj.C = C;
end