function err = checkTimingBounds(bounds)
%CHECKTIMINGBOUNDS returns an error message if the bounds are not set
%                  properly
%
%  ERR = CHECKTIMINGBOUNDS(BOUNDS) returns a string ERR if BOUNDS are not
%  suitable

    hmin    = bounds(1);
    hmax    = bounds(2);
    taumin  = bounds(3);
    taumax  = bounds(4);
    err   = 0;
    
    if ~isnumeric(bounds)
        err = 'Timing bounds must be numeric values';
    end

    if(hmin<=0 || hmax<=0)
        err='Transmission interval bounds must be positive numbers';
    elseif(taumin<0 || taumax<0)
        err='Delay bounds must be non-negative numbers';
    end
    
    if(~isnan(hmin) && ~isnan(hmax) && hmin > hmax)
        err = 'Transmission interval minimum must be less than the maximum';
    elseif(~isnan(taumin) && ~isnan(taumax) && taumin > taumax)
        err ='Transmission interval minimum must be less than the maximum';
    elseif(~isnan(taumin) && ~isnan(hmax) && hmax < taumin)
        err = 'This toolbox does not support cases where txIntvl < delay. Minimum delay must be less than maximum tranmission interval';
        
    end

