function verts=genTimingUncertaintyRegion(hmin,hmax,taumin,taumax)
%GENTIMINGUNCERTAINTYREGION generates a timing uncertainty region within
%the small-delay region given bounds on the transmission interval [HMIN,HMAX] 
%and bounds on the delays [TAUMIN,TAUMAX].
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------    
    wiggleRoom=1e-12;
    
    if taumax-taumin < wiggleRoom && hmax-hmin < wiggleRoom
        % both delay and transmission interval are constant
        verts = [hmax,taumax];
    elseif taumax-taumin < wiggleRoom
        % constant delay
        verts = [hmin taumax;
                 hmax taumax];
    elseif hmax-hmin < wiggleRoom
        % constant transmission interval
        verts = [hmax taumin;
                    hmax taumax];
    else
        if hmin >= taumax                       % rectangle shape
        verts = [hmin taumax;
                    hmin taumin;
                    hmax taumin;
                    hmax taumax];
        elseif hmin > taumin && hmax > taumax   %bottom is right, top is right
        verts = [hmin   hmin;
                    hmin   taumin;
                    hmax   taumin;
                    hmax   taumax;
                    taumax taumax];
        elseif hmin <= taumin && hmax > taumax  %bottom is left, top is right
        verts = [taumin taumin;
                    taumax taumax;
                    hmax   taumax;
                    hmax   taumin];
        elseif hmin > taumin && hmax <= taumax  %bottom is right, top is left
        verts = [hmin taumin;
                    hmin hmin;
                    hmax hmax;
                    hmax taumin];        
        elseif hmin<=taumin && hmax<=taumax     %bottom is left, top is left
        verts = [taumin taumin;
                    hmax   hmax;
                    hmax   taumin];
        end
    end
   
    