function [alfa_ht,Ab,Aconst,Eb,Econst] = reduceAlphas(alfaStack,AbStack,EbStack,h,tau)
%REDUCEALPHAS	(JNF and CH overapproximation) Reduces number of alpha terms 
%               by combining duplicate alpha terms and seperating out 
%               alpha terms which are constant.
%
%   See also JNF, JNF2CH
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------
%% First Seperate out constant alpha terms

    syms hk hktk tk
    nrNodes = size(AbStack,1);

    if h(1) - h(2) ==0
        for k=1:length(alfaStack)
            if isa(alfaStack{k},'sym')
                alfaTemp = subs(alfaStack{k},hktk,h(1)-tk,0);    %FIRST substitute 'h(1)-tk' into symbol hktk (constant sampling)
                alfaStack{k} = subs(alfaTemp,hk,h(1),0);              %THEN substitute value of hk into symbol hk (constant sampling)
            end
        end
    end

    if tau(1) - tau(2)==0
        for k=1:length(alfaStack)
            if isa(alfaStack{k},'sym')
                alfaTemp = subs(alfaStack{k},hktk,hk-tau(1),0);   %FIRST substitute 'hk-tau(1)' into symbol hktk (constant sampling)
                alfaStack{k} = subs(alfaTemp,tk,tau(1),0);             %THEN substitute tau(1) into symbol tk (constant delay)
            end
        end
    end

    % ------------------------------------------------------------------------
    %split alfaStack, AbStack, and EbStack into constant parts and varying parts
    k = 1; 
    z = 1;

    alfa_var = cell(0);                             
    Ab_var = cell(nrNodes,size(alfaStack,2));       
    Eb_var = cell(nrNodes,size(alfaStack,2));       
    
    Aconst = cell(nrNodes,1);  
    Econst = cell(nrNodes,1);
    for ell=1:nrNodes
        Aconst{ell} = zeros(size(AbStack{ell}));
        Econst{ell} = zeros(size(EbStack{ell}));        
    end
    
    
    for i = 1:size(alfaStack,2)
        if diff(alfaStack{i},hk) == 0 && ...
                diff(alfaStack{i},tk) == 0 && ...
                    diff(alfaStack{i},hktk) == 0       %constant terms    
            for ell=1:nrNodes
                Aconst{ell} = Aconst{ell} + double(alfaStack{i})*AbStack{ell,i};
                Econst{ell} = Econst{ell} + double(alfaStack{i})*EbStack{ell,i};
            end
            k = k+1;
        else                    %varying alfa terms
            alfa_var{z} = alfaStack{i};
            Ab_var(:,z) = AbStack(:,i);  % assign all node indices of AbStack to Ab_var
            Eb_var(:,z) = EbStack(:,i);  % assign all node indices of EbStack to Eb_var
            z = z+1;
        end
    end

    Ab_var = Ab_var(:,1:z-1);       %resize by index z
    Eb_var = Eb_var(:,1:z-1);       %resize by index z

%% Create alfa_ht, Ab and G_ht
% Now use alfa_var,Ab_var,Eb_var and remove duplicate alfa multipliers

    alfa_ht=cell(0);                       
    Ab=cell(nrNodes,size(Ab_var,2));       
    Eb=cell(nrNodes,size(Eb_var,2));       

    % Eliminate duplicate alfa terms in alfa_stack.  Add Ab_stack and G_stack elements if alfa's are identical
    r=1;
    for p = 1:size(alfa_var,2)
        if alfa_var{p} ~= 0    
            alfa_ht{r} = alfa_var{p};
            Ab(:,r) = Ab_var(:,p);                          % assign all node indices of Ab to Ab_var
            Eb(:,r) = Eb_var(:,p);                          % assign all node indices of Eb to Eb_var

            for m = p+1:size(alfa_var,2)                                  % compare  pth  element to rest of elements
                if alfa_var{m}-alfa_var{p} == 0                           % if symbols are equal
                    for ell=1:nrNodes
                        Ab{ell,r} = Ab{ell,r} + Ab_var{ell,m};        % add matrices of similar multipliers
                        Eb{ell,r} = Eb{ell,r} + Eb_var{ell,m};        % add matrices of similar multipliers
                    end
                    alfa_var{m} = 0;                                      % set dupiclate multiplier to zero
                end
            end
            r = r+1;
        end
    end

    Ab=Ab(:,1:r-1);           %resize to index r
    Eb=Eb(:,1:r-1);           %resize to index r
end
