function Generate_NCS_Blockdiagram(h)

  % Create figure
%    figure; clf;
%    clf
%    set(gcf, 'Position', [1 1 1080 435 ]);
%   
  % Create axis
   % h = axes;
  
  [ sc off ] = scaling_and_offset();
  
  set(h,...
    'Box'            , 'on'       ,...
    'XLim'           , [ 0.0+off(1) sc(1)+off(1) ],...
    'YLim'           , [ 0.0+off(2) sc(2)+off(2) ],...
    'XTick'          , []         ,...
    'YTick'          , []         ,...
    'DataAspectRatio', [ 1 1 1 ]  );

  hold on;
  
  % Signals
  plt(h, 'line' , [0.19 0.19], [0.48 0.62],'black');   % u_k (plant side)
  plt(h, 'arrow', [0.19 0.27], [0.62 0.62],'black');   % u_k (plant side)
  plt(h, 'arrow', [0.37 0.46], [0.62 0.62],'black');   % u^ast(t)  
  plt(h, 'arrow', [0.56 0.63], [0.62 0.62],'black');   % Plant to Sensor
  plt(h, 'arrow', [0.68 0.68], [0.79 0.69],'black');   % Clock to Sensor
  plt(h, 'line' , [0.73 0.78], [0.62 0.62],'black');   % y_k (plant side)
  plt(h, 'arrow', [0.78 0.78], [0.62 0.48],'black');   % y_k (plant side)
  plt(h, 'line' , [0.78 0.78], [0.35 0.21],'black');   % y_k (controller side)
  plt(h, 'arrow', [0.78 0.60], [0.21 0.21],'black');   % y_k (controller side)
  %plt(h, 'arrow', [0.65 0.60], [0.15 0.15],'black');   % r_k
  plt(h, 'line' , [0.43 0.19], [0.21 0.21],'black');   % u_k (controller side)
  plt(h, 'arrow', [0.19 0.19], [0.21 0.35],'black');   % u_k (controller side)
  
  % Signal Annotations
  %txt(h, '$u_k$'        , [0.18 0.61 0.10 0.10], 0, 0, 'black');  % u_k
  %txt(h, '$u^{\ast}(t)$', [0.37 0.61 0.10 0.10], 0, 'black');  % u^ast(t)
  %txt(h, '$y_k$'        , [0.71 0.61 0.10 0.10], 0, 0, 'black');  % y_k
  %txt(h, '$r_k = 0$'    , [0.63 0.10 0.13 0.11], 0, 0, 'black');  % r_k
  
  % Blocks
  txt(h, 'ZOH'       , [0.27 0.56 0.10 0.13], 1, 1, 'black');   % ZOH
  txt(h, 'Plant'     , [0.46 0.56 0.10 0.13], 1, 1, [0 0.8 0]);    % Plant
  txt(h, 'Sensor'    , [0.63 0.56 0.10 0.13], 1, 1, 'black');   % Sensor
  txt(h, 'Clock'     , [0.63 0.79 0.10 0.13], 1, 1, 'black');   % Clock
  txt(h, 'Controller', [0.43 0.15 0.17 0.13], 1, 1, 'blue');   % Controller

  % Communication Network
  plt(h, 'dline', [0.10 0.87], [0.48 0.48],'red');
  plt(h, 'dline', [0.10 0.87], [0.35 0.35],'red');
  txt(h, 'Communication Network', [0.12 0.35 0.74 0.13], 0, 1,'red');
  %txt(h, '$\tau^{ca}$'                    , [0.15 0.37 0.10 0.10], 0);
  %txt(h, '$\tau^{sc}$'                    , [0.73 0.37 0.10 0.10], 0);
  
  hold off;
  
% end function


% -- Support functions --
function [ scale, offset ] = scaling_and_offset()

  scale  = 1.5*[ 2.7 1.0 ];
  offset = [ -0.6 -0.1 ];

% end function


function plt(h, type, x, y,color)

  lineWidth = 1;
  % Get and apply global scaling and offset
  [ sc off ] = scaling_and_offset();
  
  x = sc(1)*x+off(1);
  y = sc(2)*y+off(2);

  % Arrowhead properties
  a = 3*0.030;
  b = 17.5*(pi/180);

  switch type
    
    case 'line'
      plot(x, y, ...
        'Color', color, ...
        'LineWidth' , lineWidth);
      
    case 'dline'
      plot(x, y,...
        'Color'     , color,...
        'LineStyle' , '--', ...
        'LineWidth' , lineWidth);
      
    case 'arrow'
      plot(x, y,...
        'Color', color, ...
        'LineWidth' , lineWidth);
      
      % Draw the arrowhead
      dx = x(2)-x(1);
      dy = y(2)-y(1);
      
      th = atan2(dy,dx);
      L  = sqrt(dx^2+dy^2);
      
      patch([ x(2)-a*cos(th-b) x(2) x(2)-a*cos(th+b) ],...
            [ y(2)-a*sin(th-b) y(2) y(2)-a*sin(th+b) ],...
            color);
  end
      
% end function


function txt(h, str, pos, border, addFill, color)

  % Get and apply global scaling and offset
  [ sc off ] = scaling_and_offset();
    
  x      = sc(1)*pos(1)+off(1);
  y      = sc(2)*pos(2)+off(2);
  width  = sc(1)*pos(3);
  height = sc(2)*pos(4);

  % Border yes/no
  if (border == 1)
    plot([ x x+width x+width  x        x ],...
         [ y y       y+height y+height y ],...
         'Color', color,'Linewidth',1);         
  end
  
  if (addFill == 1)
    fill([ x x+width x+width  x        x ],...
         [ y y       y+height y+height y ],...
         color,'EdgeColor','none');         
    alpha(0.2);
  end
  

  % Draw the textbox
%    'Interpreter'         , 'latex'   ,...
  text(x+width/2, y+height/2, str,...
     'VerticalAlignment'   , 'middle'  ,...
     'HorizontalAlignment' , 'center'  ,...
     'FontSize'            , 8         ,...
     'Color'           , 'white');
% end function