function [pt timingDist tPoly percentIncluded] = genUncertainPolyFromMeasurementData(measData,delta,res)
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------

    if(size(measData,2)==2) %measurement data is raw and distribution needs to be computed
        hk  = measData(:,1);
        tau = measData(:,2);
        %------------------------------------------------------------
        % Determine histogram
        
        % sort values
        val = [hk tau];
        val_sort = sortrows(val);
        [pts,index] = unique(val_sort,'rows');
        index = [0;index];

        rep = zeros(length(pts),1);
        for i = 1:length(index)-1
            rep(i) = index(i+1)-index(i);
        end

        totalData = size(val,1);

        rep = rep./totalData*100;               %normalize
        pts_sort = sortrows([pts rep],-3);

    elseif(size(measData,2)==3) %distribution already computed
        pts_sort=measData;
    elseif(isempty(measData))
        pt.sq=[];
        pt.color=[];
        timingDist=[];
        tPoly=[];
        percentIncluded=0;
        return;
    end
    
    %------------------------------------------------------------
    % Eliminate points based on threshold
    
    if all(pts_sort(:,3)<delta)
        pt.sq=[];
        pt.color=[];
        timingDist=[];
        tPoly=[];
        percentIncluded=0;
        return;
    else
        cutoff = delta;
    end

    cutoff_index=find(pts_sort(:,3)>=cutoff,1,'last');
    timingDist = pts_sort(1:cutoff_index,:);    
    
    
%------------------------------------------------------------
    % Add boxing to account for resolution
    l = size(timingDist,1);
    % build boxes around selection points
    sel2 = [];
    
    %plot(timingDist(:,1),timingDist(:,2),'or')

    % set colorbar, scaled to repetitions
    if delta==0 && size(timingDist,1)>1 
        set(gca,'CLim',[timingDist(size(timingDist,1),3) timingDist(1,3)]);
    end
    pt=cell(0);
 
    for i = 1: l
        pt1 = [timingDist(i,1)+0.5*res timingDist(i,2)+0.5*res];
        pt2 = [timingDist(i,1)-0.5*res timingDist(i,2)+0.5*res];
        pt3 = [timingDist(i,1)-0.5*res timingDist(i,2)-0.5*res];
        pt4 = [timingDist(i,1)+0.5*res timingDist(i,2)-0.5*res];

        pt.sq{i}= [pt1; pt2; pt3; pt4];
        pt.color{i} = timingDist(i,3);
        sel2 = [sel2;pt.sq{i}];
        
    end
    
    %plot([0,0],[1,1],'g');
    %place marker on color legend indicating threshold
    %ticks = get(c, 'YTick');
    %ticks = sort([ticks cutoff]);
    %set(c, 'YTick', ticks);
    
    % only take vertices of convex hull
    sel_hk = sel2(:,1);
    sel_tau = sel2(:,2);
    sel_indx = convhull(sel_hk,sel_tau,'simplify',true);
    tPoly = [sel_hk(sel_indx),sel_tau(sel_indx)];
    tPoly=tPoly(1:end-1,:); %last point is same as first, so remove it
    
    %percentIncluded based on how many data points are within the tPoly
    percentIncluded=0;
    for ctr=1:size(timingDist,1)
        if inpolygon(timingDist(ctr,1), timingDist(ctr,2), tPoly(:,1), tPoly(:,2))
            percentIncluded=percentIncluded+timingDist(ctr,3);
        end
    end
    
end