function varargout = ncsEditor(varargin)
%NCSEDITOR  Opens the graphical interface used to create an NCS object 
%           used in the networked control systems toolbox.  NCS objects can
%           be imported and exported via the editor for easy model
%           management.  Also stability analysis can be directly performed
%           via the different analysis tools which are included with the 
%           toolbox.
%
%   See also NCS
%
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------

% Last Modified by GUIDE v2.5 26-Nov-2013 13:41:25

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @ncsEditor_OpeningFcn, ...
                   'gui_OutputFcn',  @ncsEditor_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT




% --- Executes just before ncsEditor is made visible.
function ncsEditor_OpeningFcn(hObject, eventdata, handles, varargin) %#ok<*INUSL>
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to ncsEditor (see VARARGIN)

% Choose default command line output for ncsEditor
handles.output = hObject;
set(handles.ncsGui,'UserData',0);
% Update handles structure
guidata(hObject, handles);

% UIWAIT makes ncsEditor wait for user response (see UIRESUME)
% uiwait(handles.ncsEditor);


% --- Outputs from this function are returned to the command line.
function varargout = ncsEditor_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;


% --------------------------------------------------------------------
function File_Callback(hObject, eventdata, handles) %#ok<*INUSD>
% hObject    handle to File (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --------------------------------------------------------------------
function Export_Callback(hObject, eventdata, handles)
% hObject    handle to Export (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

    ncs = buildNcs(handles);
    if isempty(ncs) == 0

        prompt = {'Enter NCS name:'};
        title = 'Save NCS';
        lines = 1;

        if strcmp(strtok(get(handles.A,'String'),'.'),get(handles.A,'String'))==0
            def{1} = strtok(get(handles.A,'String'),'.');
        elseif strcmp(strtok(get(handles.B,'String'),'.'),get(handles.B,'String'))==0
            def{1} = strtok(get(handles.B,'String'),'.');
        elseif strcmp(strtok(get(handles.C,'String'),'.'),get(handles.C,'String'))==0
            def{1} = strtok(get(handles.C,'String'),'.');    
        else
            def{1} = 'my_ncs';
        end
        answer = inputdlg(prompt, title, lines, def);
        if isempty(answer) == 0
            assignin('base', answer{1}, ncs);
        end
    else
        clear ncs
    end

% --------------------------------------------------------------------
function Import_Callback(hObject, eventdata, handles) %#ok<*DEFNU>
% hObject    handle to Import (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

    S= evalin('base','whos');
    NcsNames = cell(0);
    for ctr = 1:length(S)
        if strcmp(S(ctr).class,'ncs')
           NcsNames{ctr} = S(ctr).name;
        end
    end
    NcsNames(cellfun(@isempty,NcsNames)) = [];
    
    if isempty(NcsNames) == 0
        [Selection,OK] = listdlg('ListString',NcsNames,'SelectionMode','single', ...
                         'Name','Load NCS', ...
                         'PromptString','Select NCS from Workspace:', 'ListSize', [200 min(max(50,length(S)*17),300)]);
        if OK == 1
            ncsName = char(NcsNames(Selection));
            
            %Import Plant Data
            set(handles.A,'String',strcat(ncsName,'.getPlant.A'));
            set(handles.B,'String',strcat(ncsName,'.getPlant.B'));
            set(handles.C,'String',strcat(ncsName,'.getPlant.C'));
            
            %Import Controller Data
            %uWired = evalin('base',char(strcat(ncsName,'.uWired')));
            %if isempty(uWired)
            %    set(handles.uWiredCheckBox,'Value',0);
            %else
            %    set(handles.uWiredCheckBox,'Value',uWired);
            %end
            
            ctrlType = evalin('base',char(strcat(ncsName,'.getController.ctrlType')));

            
            switch ctrlType
                case 'Static Feedback'
                    set(handles.controllerType,'Value',1);
                    controllerType_Callback(handles.controllerType,eventdata,handles);
                    
                    nrControllers = length(evalin('base',char(strcat(ncsName,'.getController.K'))));
                    
                    if nrControllers>1
                       set(handles.AcText,'String','K1:') 
                    else
                        set(handles.AcText,'String','K:')
                    end
                    
                    Ac = cell(1,nrControllers);
                    Bc = cell(1,nrControllers);
                    Cc = cell(1,nrControllers);
                    Dc = cell(1,nrControllers);
                    Ec = cell(1,nrControllers);
                    
                    %reset current values
                    set(handles.Ac,'String',strcat(ncsName,'.getController.K{1}'));
                    set(handles.Bc,'String','');
                    set(handles.Cc,'String','');
                    set(handles.Dc,'String','');
                    set(handles.Ec,'String','');
                    
                    for ctr=1:nrControllers
                        Ac{ctr} = strcat(ncsName,'.getController.K{',num2str(ctr),'}');
                    end
                    
                case 'D-LTI Observer-Based Feedback'
                    set(handles.controllerType,'Value',4);
                    controllerType_Callback(handles.controllerType,eventdata,handles);
                    
                    nrControllers = length(evalin('base',char(strcat(ncsName,'.getController.A'))));
                    Ac = cell(1,nrControllers);
                    Bc = cell(1,nrControllers);
                    Cc = cell(1,nrControllers);
                    Dc = cell(1,nrControllers);
                    Ec = cell(1,nrControllers);
                    
                    if nrControllers>1
                       set(handles.AcText,'String','A1:') 
                       set(handles.BcText,'String','B1:')
                       set(handles.CcText,'String','C1:')
                       set(handles.DcText,'String','K1:')
                       set(handles.EcText,'String','L1:')
                    else
                        set(handles.AcText,'String','A:')
                        set(handles.BcText,'String','B:')
                        set(handles.CcText,'String','C:')
                        set(handles.DcText,'String','K:')
                        set(handles.EcText,'String','L:')
                    end
                    
                    set(handles.Ac,'String',strcat(ncsName,'.getController.A{1}'));
                    set(handles.Bc,'String',strcat(ncsName,'.getController.B{1}'));
                    set(handles.Cc,'String',strcat(ncsName,'.getController.C{1}'));
                    set(handles.Dc,'String',strcat(ncsName,'.getController.K{1}'));
                    set(handles.Ec,'String',strcat(ncsName,'.getController.L{1}'));
                    
                    for ctr=1:nrControllers
                        Ac{ctr} = strcat(ncsName,'.getController.A{',num2str(ctr),'}');
                        Bc{ctr} = strcat(ncsName,'.getController.B{',num2str(ctr),'}');
                        Cc{ctr} = strcat(ncsName,'.getController.C{',num2str(ctr),'}');
                        Dc{ctr} = strcat(ncsName,'.getController.K{',num2str(ctr),'}');
                        Ec{ctr} = strcat(ncsName,'.getController.L{',num2str(ctr),'}');
                    end
                    
                case {'C-LTI Dynamic Feedback','C-LTI Dynamic Feedback (uWired)'}
                    set(handles.controllerType,'Value',2);
                    controllerType_Callback(handles.controllerType,eventdata,handles);
    
                    nrControllers = length(evalin('base',char(strcat(ncsName,'.getController.A'))));
                    Ac = cell(1,nrControllers);
                    Bc = cell(1,nrControllers);
                    Cc = cell(1,nrControllers);
                    Dc = cell(1,nrControllers);
                    Ec = cell(1,nrControllers);

                                        
                    if nrControllers>1
                       set(handles.AcText,'String','A1:') 
                       set(handles.BcText,'String','B1:')
                       set(handles.CcText,'String','C1:')
                       set(handles.DcText,'String','D1:')
                    else
                        set(handles.AcText,'String','A:')
                        set(handles.BcText,'String','B:')
                        set(handles.CcText,'String','C:')
                        set(handles.DcText,'String','D:')
                    end
                    
                    set(handles.Ac,'String',strcat(ncsName,'.getController.A{1}'));
                    set(handles.Bc,'String',strcat(ncsName,'.getController.B{1}'));
                    set(handles.Cc,'String',strcat(ncsName,'.getController.C{1}'));
                    set(handles.Dc,'String',strcat(ncsName,'.getController.D{1}'));
                    set(handles.Ec,'String','');
                    
                    for ctr=1:nrControllers
                        Ac{ctr} = strcat(ncsName,'.getController.A{',num2str(ctr),'}');
                        Bc{ctr} = strcat(ncsName,'.getController.B{',num2str(ctr),'}');
                        Cc{ctr} = strcat(ncsName,'.getController.C{',num2str(ctr),'}');
                        Dc{ctr} = strcat(ncsName,'.getController.D{',num2str(ctr),'}');
                    end
                    
                case {'D-LTI Dynamic Feedback'}
                    set(handles.controllerType,'Value',3);
                    controllerType_Callback(handles.controllerType,eventdata,handles);
                    
                    nrControllers = length(evalin('base',char(strcat(ncsName,'.getController.A'))));
                    Ac = cell(1,nrControllers);
                    Bc = cell(1,nrControllers);
                    Cc = cell(1,nrControllers);
                    Dc = cell(1,nrControllers);
                    Ec = cell(1,nrControllers);

                    if nrControllers>1
                       set(handles.AcText,'String','A1:') 
                       set(handles.BcText,'String','B1:')
                       set(handles.CcText,'String','C1:')
                       set(handles.DcText,'String','D1:')
                    else
                        set(handles.AcText,'String','A:')
                        set(handles.BcText,'String','B:')
                        set(handles.CcText,'String','C:')
                        set(handles.DcText,'String','D:')
                    end
                    
                    set(handles.Ac,'String',strcat(ncsName,'.getController.A{1}'));
                    set(handles.Bc,'String',strcat(ncsName,'.getController.B{1}'));
                    set(handles.Cc,'String',strcat(ncsName,'.getController.C{1}'));
                    set(handles.Dc,'String',strcat(ncsName,'.getController.D{1}'));
                    set(handles.Ec,'String','');
                   
                    for ctr=1:nrControllers
                        Ac{ctr} = strcat(ncsName,'.getController.A{',num2str(ctr),'}');
                        Bc{ctr} = strcat(ncsName,'.getController.B{',num2str(ctr),'}');
                        Cc{ctr} = strcat(ncsName,'.getController.C{',num2str(ctr),'}');
                        Dc{ctr} = strcat(ncsName,'.getController.D{',num2str(ctr),'}');
                    end
                    
            end
            
            %clear any stored data
            set(handles.Ac,'UserData',Ac);
            set(handles.Bc,'UserData',Bc);
            set(handles.Cc,'UserData',Cc);
            set(handles.Dc,'UserData',Dc);
            set(handles.Ec,'UserData',Ec);

            if strcmp(ctrlType,'C-LTI Dynamic Feedback (uWired)')
                set(handles.uWiredCheckBox,'Value',1);
            else
                set(handles.uWiredCheckBox,'Value',0);
            end
                        
            if nrControllers>1
                set(handles.nodeDepCheckBox,'Value',1);
                set(handles.ctlNodeSelect,'Enable','on');
            else
                set(handles.nodeDepCheckBox,'Value',0);
                set(handles.ctlNodeSelect,'Enable','off');
            end
            set(handles.ctlNodeSelect,'Value',1); 
            set(handles.ctlNodeSelect,'UserData',1); %reset last node
            
            
            %Import Network data 
            hmin = evalin('base',strcat(ncsName,'.getNetwork.txIntvls(1)'));
            set(handles.hmin,'String',num2str(hmin));
            set(handles.hmin,'ForegroundColor','black')
            hmax = evalin('base',strcat(ncsName,'.getNetwork.txIntvls(2)'));
            set(handles.hmax,'String',num2str(hmax));
            set(handles.hmax,'ForegroundColor','black')
            taumin = evalin('base',strcat(ncsName,'.getNetwork.delays(1)'));
            set(handles.taumin,'String',num2str(taumin));
            set(handles.taumin,'ForegroundColor','black')
            taumax = evalin('base',strcat(ncsName,'.getNetwork.delays(2)'));
            set(handles.taumax,'String',num2str(taumax));
            set(handles.taumax,'ForegroundColor','black')
            
            tPoly       = evalin('base',strcat(ncsName,'.getNetwork.tPoly'));
            timingDist  = evalin('base',strcat(ncsName,'.getNetwork.timingDist'));
            set(handles.refineButton,'UserData',{tPoly,timingDist});
            
            if(~isempty(tPoly))
                %disable user from changing boxes, values are based on tPoly
                set(handles.hmin,'Enable','Off');
                set(handles.hmax,'Enable','Off');
                set(handles.taumin,'Enable','Off');
                set(handles.taumax,'Enable','Off');
            else
                %no polygon, so enable user to modify values
                set(handles.hmin,'Enable','On');
                set(handles.hmax,'Enable','On');
                set(handles.taumin,'Enable','On');
                set(handles.taumax,'Enable','On');       
            end
            set(handles.refineButton,'Enable','On');
            
            delta = evalin('base',strcat(ncsName,'.getNetwork.maxDrops'));
            set(handles.delta,'String',num2str(delta));
            set(handles.delta,'ForegroundColor','black')
            
            nodesU = evalin('base',strcat(ncsName,'.getNetwork.nodesU'));
            nodesY = evalin('base',strcat(ncsName,'.getNetwork.nodesY'));
            
            B = evalin('base',strcat(ncsName,'.getPlant.B'));
            C = evalin('base',strcat(ncsName,'.getPlant.C'));
            if length(nodesU) == 1 && length(nodesY) == 1 && sum(sum(nodesU{1})) + sum(sum(nodesY{1})) == size(B,2)+size(C,2)
                set(handles.ctlNodeSelect,'String',{'1'});
                set(handles.sharedCheckBox,'Value',0);
                set(handles.nodeList,'String','');
                sharedCheckBox_Callback(handles.sharedCheckBox, eventdata, handles)
                set(handles.nodeDepCheckBox,'Value',0);
            else                
                
                nodeList = cell(0);
                for ctr = 1:length(nodesU)
                    if ctr <= 9
                        nodeList{ctr} = ['node' num2str(ctr) '     '];
                    else
                        nodeList{ctr} = ['node' num2str(ctr) '    '];
                    end

                    for ctr2=1:size(nodesU{ctr},1)
                        if nodesU{ctr}(ctr2,ctr2) == 1
                            if ctr2 <= 9
                                nodeList{ctr} = [nodeList{ctr} 'u0' num2str(ctr2) ','];
                            else
                                nodeList{ctr} = [nodeList{ctr} 'u' num2str(ctr2) ','];
                            end
                        end
                    end

                    for ctr2=1:size(nodesY{ctr},1)
                        if nodesY{ctr}(ctr2,ctr2) == 1
                            if ctr2 <= 9
                                nodeList{ctr} = [nodeList{ctr} 'y0' num2str(ctr2) ',']; 
                            else
                                nodeList{ctr} = [nodeList{ctr} 'y' num2str(ctr2) ',']; 
                            end
                        end
                    end
                    if nodeList{ctr}(end) == ',' 
                        nodeList{ctr} = nodeList{ctr}(1:end-1);   %remove last comma
                    end
                end
                
                nodeString = cell(1,length(nodesU));
                for ctr = 1:length(nodesU)
                   nodeString{ctr} = num2str(ctr); 
                end
                set(handles.ctlNodeSelect,'String',nodeString)
                
                set(handles.nodeList,'String',nodeList);
                set(handles.sharedCheckBox,'Value',1);
                set(handles.nodeList,'Value',1);
                set(handles.pushbutton4,'Enable','on');
                set(handles.nodeList,'Enable','on');
                set(handles.protocolChoice,'Enable','on');
            end

            protocol = evalin('base',strcat(ncsName,'.getNetwork.protocol'));

            if strcmp(protocol,'RR')
                set(handles.protocolChoice,'Value',1); 
            elseif strcmp(protocol,'TOD')
                set(handles.protocolChoice,'Value',2); 
            end

            quantizer = evalin('base',strcat(ncsName,'.getNetwork.quantizer'));

            if strcmp(quantizer,'none') || isempty(quantizer)
                set(handles.quantizer,'Value',1); 
            elseif strcmp(quantizer,'log')
                set(handles.protocolChoice,'Value',2); 
            elseif strcmp(quantizer,'uniform')
                set(handles.quantizer,'Value',3); 
            end        
        end
    else
        msgbox('No NCS variables in workspace')
    end


% --------------------------------------------------------------------
function Close_Callback(hObject, eventdata, handles)
% hObject    handle to Close (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
    button = questdlg('Are you sure you want to quit?  All un-exported data will be lost.','NCS Editor','Ok','Cancel','Cancel'); 
    if strcmp(button,'Ok')
        delete(handles.ncsGui);
    end

% --- Executes during object creation, after setting all properties.
function delta_CreateFcn(hObject, eventdata, handles)
% hObject    handle to delta (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


function ncs1 = buildNcs(handles)


    contents = cellstr(get(handles.ctlNodeSelect,'String'));
    lastNode = str2double(contents{get(handles.ctlNodeSelect,'Value')});
 
    %write last piece of data into 'userData' field

    AcData = get(handles.Ac,'UserData');            % retrive saved data
    AcData{lastNode} = get(handles.Ac,'String');    % store new input
    set(handles.Ac,'UserData',AcData)               % save into handles.Ac       

    BcData = get(handles.Bc,'UserData');            % retrive saved data
    BcData{lastNode} = get(handles.Bc,'String');    % store new input
    set(handles.Bc,'UserData',BcData)               % save into handles.Bc       

    CcData = get(handles.Cc,'UserData');            % retrive saved data
    CcData{lastNode} = get(handles.Cc,'String');    % store new input
    set(handles.Cc,'UserData',CcData)               % save into handles.Cc       

    DcData = get(handles.Dc,'UserData');            % retrive saved data
    DcData{lastNode} = get(handles.Dc,'String');    % store new input
    set(handles.Dc,'UserData',DcData)               % save into handles.Dc          

    EcData = get(handles.Ec,'UserData');            % retrive saved data
    EcData{lastNode} = get(handles.Ec,'String');    % store new input
    set(handles.Ec,'UserData',EcData)               % save into handles.Ec 

    %-------------------------------------------------------------------    
    %check A,B,C conditions
    try
        A = evalin('base',char(get(handles.A,'String')));
    catch %#ok<*CTCH>
        msgbox('Invalid value for input field A entered','NCS Toolbox Error','error')
        ncs1 =[];
        return
    end

     if isnumeric(A) == 0
         msgbox('Input field A must be a matrix of type double','NCS Toolbox Error','error')
         ncs1 =[];
        return
     elseif size(A,1) ~= size(A,2)
        msgbox('Input field A must be a square matrix','NCS Toolbox Error','error')
        ncs1 =[];
        return
     end
    
    
    try
        B = evalin('base',char(get(handles.B,'String')));
    catch
        msgbox('Invalid value for input field B entered','NCS Toolbox Error','error')
        ncs1 =[];
        return
    end
    
     if isnumeric(B) == 0
         msgbox('Input field B must be a matrix of type double','NCS Toolbox Error','error')
         ncs1 =[];
        return
     elseif size(A,2) ~= size(B,1)
        msgbox('Number of rows of A must match number of rows of B','NCS Toolbox Error','error')
        ncs1 =[];
        return
     end
     
    try
        C = evalin('base',char(get(handles.C,'String')));
    catch
        msgbox('Invalid value for input field C entered','NCS Toolbox Error','error')
        ncs1 =[];
        return
    end
    
     if isnumeric(C) == 0
         msgbox('Input field C must be a matrix of type double','NCS Toolbox Error','error')
         ncs1 =[];
        return
     elseif size(A,2) ~= size(C,2)
        msgbox('Number of columns of A must match number of columns of C','NCS Toolbox Error','error')
        ncs1 =[];
        return
     end 
    %------------------------------------------------------------------- 
    %-------------------------------------------------------------------    
    %check Controller conditions    
    
    contents = get(handles.controllerType,'String');
    controllerType = contents{get(handles.controllerType,'Value')};
    uWired = get(handles.uWiredCheckBox,'Value');
    
    if get(handles.nodeDepCheckBox,'Value') == 1
        nrControllerNodes = length(get(handles.ctlNodeSelect,'String'));
    else
        nrControllerNodes = 1;
    end
    
    K=cell(nrControllerNodes,1);
    L=cell(nrControllerNodes,1);
    Ac=cell(nrControllerNodes,1);
    Bc=cell(nrControllerNodes,1);
    Cc=cell(nrControllerNodes,1);
    Dc=cell(nrControllerNodes,1);
    
    switch controllerType
        %-------------------------------------------------------------------    
        %Static Feedback checks
        case 'Static Feedback'
            for ctr=1:nrControllerNodes
                try
                    K{ctr} = evalin('base',AcData{ctr});
                catch
                    msgbox('Invalid value for input field K entered','NCS Toolbox Error','error')
                    ncs1 =[];
                    return
                end   
            end
            
            if length(K) ~= 1 && length(K) ~= length(get(handles.nodeList,'String'))
                msgbox('K must be a single matrix or a cell whose length is equal to the number of nodes','NCS Toolbox Error','error')
                ncs1 =[];
                return
            end 
            
            for ctr = 1:nrControllerNodes
                if isnumeric(K{ctr}) == 0
                     msgbox('Input field K must either a matrix of type double or a cell of matrices type double','NCS Toolbox Error','error')
                     ncs1 =[];
                    return
                elseif isempty(K{ctr}) == 0 && size(B,2) ~= size(K{ctr},1)
                    msgbox('Number of columns of B must match number of rows of K','NCS Toolbox Error','error')
                    ncs1 =[];
                    return
                elseif isempty(K{ctr}) == 0 && size(C,1) ~= size(K{ctr},2)
                    msgbox('Number of columns of K must match number of rows of A','NCS Toolbox Error','error')
                    ncs1 =[];
                    return
                end
            end
        %-------------------------------------------------------------------    
        %Obsver-based Feedback checks
        case 'D-LTI  Observer-Based Feedback'

            for ctr=1:nrControllerNodes
                try
                   Ac{ctr}=evalin('base',AcData{ctr});
                catch
                    msgbox('Invalid value for the controller A matrix entered','NCS Toolbox Error','error')
                    ncs1 =[];
                    return
                end
            end
            
            for ctr=1:nrControllerNodes
                try
                    Bc{ctr}=evalin('base',BcData{ctr});
                catch
                    msgbox('Invalid value for the controller B matrix entered','NCS Toolbox Error','error')
                    ncs1 =[];
                    return
                end
            end
            
            for ctr=1:nrControllerNodes
                try
                    Cc{ctr}=evalin('base',CcData{ctr});
                catch
                    msgbox('Invalid value for the controller C matrix entered','NCS Toolbox Error','error')
                    ncs1 =[];
                    return
                end
            end
         
            
            for ctr=1:nrControllerNodes
                try
                    K{ctr}=evalin('base',DcData{ctr});
                catch
                    msgbox('Invalid value for the controller K matrix entered','NCS Toolbox Error','error')
                    ncs1 =[];
                    return
                end   
            end

            for ctr=1:nrControllerNodes
                try
                    L{ctr}=evalin('base',EcData{ctr});
                catch
                    msgbox('Invalid value forthe controller L matrix entered','NCS Toolbox Error','error')
                    ncs1 =[];
                    return
                end   
            end
            
            if length(K) ~= length(L)
                    msgbox('Number of K matrices does not equal number of L matrices','NCS Toolbox Error','error')
                    ncs1 =[];
                    return
            end
            
            if length(L) ~= 1 && length(K) ~= length(get(handles.nodeList,'String'))
                msgbox('K and L must be single matrices or cells whose length is equal to the number of nodes','NCS Toolbox Error','error')
                ncs1 =[];
                return
            end 
            
            
            for ctr = 1:nrControllerNodes
                 
                if isnumeric(Ac{ctr}) == 0
                     msgbox('The controller A matrix must be a matrix of type double','NCS Toolbox Error','error')
                     ncs1 =[];
                    return
                 elseif size(Ac{ctr},1) ~= size(Ac{ctr},2)
                    msgbox('The controller A matrix must be a square matrix','NCS Toolbox Error','error')
                    ncs1 =[];
                    return
                elseif size(Ac{1},1) ~= size(Ac{ctr},1)
                    msgbox('All controller A matrices must be the same dimension','NCS Toolbox Error','error')
                    ncs1 =[];
                    return
                 end

                 if isnumeric(Bc{ctr}) == 0
                     msgbox('The controller B matrix must be a matrix of type double','NCS Toolbox Error','error')
                     ncs1 =[];
                    return
                 elseif size(Ac{ctr},2) ~= size(Bc{ctr},1)
                    msgbox('Number of columns of the controller A matrix must match number of rows of the controller B matrix ','NCS Toolbox Error','error')
                    ncs1 =[];
                    return
                 end

                 if isnumeric(Cc{ctr}) == 0
                     msgbox('The controller C matrix  must be a matrix of type double','NCS Toolbox Error','error')
                     ncs1 =[];
                    return
                 elseif size(Ac{ctr},2) ~= size(Cc{ctr},2)
                    msgbox('Number of columns of the controller A matrix must match number of columns of the controller C matrix ','NCS Toolbox Error','error')
                    ncs1 =[];
                    return
                 end                
                
                if isnumeric(K{ctr}) == 0
                     msgbox('The controller K matrix must either a matrix of type double or a cell of matrices type double','NCS Toolbox Error','error')
                     ncs1 =[];
                    return
                elseif isempty(K{ctr}) == 0 && size(K{ctr},1) ~= size(B,2) 
                    msgbox('Number of columns of the plant B matrix must match number of rows of the controller K matrix','NCS Toolbox Error','error')
                    ncs1 =[];
                    return
                elseif isempty(K{ctr}) == 0 && size(K{ctr},2) ~= size(A,1)+size(B,2)
                    msgbox('Number of columns of the controller K matrix must match number of rows of the plant A matrix','NCS Toolbox Error','error')
                    ncs1 =[];
                    return
                end

                if isnumeric(L{ctr}) == 0
                     msgbox('The controller L matrix must either a matrix of type double or a cell of matrices type double','NCS Toolbox Error','error')
                     ncs1 =[];
                    return
                elseif isempty(L{ctr}) == 0 && size(C,2) ~= size(L{ctr},1)
                    msgbox('Number of columns of the plant C matrix  must match number of rows of the controller L matrix ','NCS Toolbox Error','error')
                    ncs1 =[];
                    return
                elseif isempty(L{ctr}) == 0 && size(A,1) ~= size(L{ctr},1)
                    msgbox('Number of columns of the controller L matrix  must match number of rows of the plant C matrix ','NCS Toolbox Error','error')
                    ncs1 =[];
                    return
                end
            end
            
           if length(Ac) ~= length(Bc) || length(Ac) ~= length(Cc)
                msgbox('Number of controller matrices A,B,C are not equal','NCS Toolbox Error','error')
                ncs1 =[];
                return
           end
           
           if length(Ac) ~= 1 && length(Ac) ~= length(get(handles.nodeList,'String'))
                msgbox('The controller A,B,C,D matrices must be a set of single matrices or a set of cells whose length is equal to the number of nodes','NCS Toolbox Error','error')
                ncs1 =[];
                return
           end  
            
        %-------------------------------------------------------------------    
        %Dynamic Feedback checks    
        case {'C-LTI  Dynamic Feedback', 'D-LTI  Dynamic Feedback'}
            
            for ctr=1:nrControllerNodes
                try
                   Ac{ctr} = evalin('base',AcData{ctr});
                catch
                    msgbox('Invalid value for the controller A matrix entered','NCS Toolbox Error','error')
                    ncs1 =[];
                    return
                end
            end
            
            for ctr=1:nrControllerNodes
                try
                    Bc{ctr} = evalin('base',BcData{ctr});
                catch
                    msgbox('Invalid value for the controller B matrix entered','NCS Toolbox Error','error')
                    ncs1 =[];
                    return
                end
            end
            
            for ctr=1:nrControllerNodes
                try
                    Cc{ctr} = evalin('base',CcData{ctr});
                catch
                    msgbox('Invalid value for the controller C matrix entered','NCS Toolbox Error','error')
                    ncs1 =[];
                    return
                end
            end
            
           for ctr=1:nrControllerNodes
              try
                  Dc{ctr} = evalin('base',DcData{ctr});
              catch
                  msgbox('Invalid value for the controller D matrix entered','NCS Toolbox Error','error')
                  ncs1 =[];
                  return
              end
           end
           
           if length(Ac) ~= length(Bc) || length(Ac) ~= length(Cc) || length(Ac) ~= length(Dc)
                msgbox('Number of controller matrices A,B,C are not equal','NCS Toolbox Error','error')
                ncs1 =[];
                return
           end
           
           if length(Ac) ~= 1 && length(Ac) ~= length(get(handles.nodeList,'String'))
                msgbox('The controller matrices A,B,C,D must be a set of single matrices or a set of cells whose length is equal to the number of nodes','NCS Toolbox Error','error')
                ncs1 =[];
                return
           end  
           
           
           for ctr = 1:nrControllerNodes
                
                 if isnumeric(Ac{ctr}) == 0
                     msgbox('The controller A matrix must be a matrix of type double','NCS Toolbox Error','error')
                     ncs1 =[];
                    return
                 elseif size(Ac{ctr},1) ~= size(Ac{ctr},2)
                    msgbox('The controller A matrix must be a square matrix','NCS Toolbox Error','error')
                    ncs1 =[];
                    return
                elseif size(Ac{1},1) ~= size(Ac{ctr},1)
                    msgbox('All controller A matrices must be the same dimension','NCS Toolbox Error','error')
                    ncs1 =[];
                    return
                 end

                 if isnumeric(Bc{ctr}) == 0
                     msgbox('The controller B matrix  must be a matrix of type double','NCS Toolbox Error','error')
                     ncs1 =[];
                    return
                 elseif size(Ac{ctr},2) ~= size(Bc{ctr},1)
                    msgbox('Number of columns of the controller A matrix must match number of rows of the controller B matrix','NCS Toolbox Error','error')
                    ncs1 =[];
                    return
                 end

                 if isnumeric(Cc{ctr}) == 0
                     msgbox('The controller C matrix must be a matrix of type double','NCS Toolbox Error','error')
                     ncs1 =[];
                    return
                 elseif size(Ac{ctr},2) ~= size(Cc{ctr},2)
                    msgbox('Number of columns of the controller A matrix must match number of columns of the controller C matrix','NCS Toolbox Error','error')
                    ncs1 =[];
                    return
                 end

                 if isnumeric(Dc{ctr}) == 0
                     msgbox('The controller D matrix must be a matrix of type double','NCS Toolbox Error','error')
                     ncs1 =[];
                    return
                 elseif size(Bc{ctr},2) ~= size(Dc{ctr},2)
                    msgbox('Number of columns of The controller B matrix must match number of columns of the controller D matrix','NCS Toolbox Error','error')
                    ncs1 =[];
                    return
                elseif size(Cc{ctr},1) ~= size(Dc{ctr},1)
                    msgbox('Number of rows of the controller C matrix must match number of rows of the controller D matrix','NCS Toolbox Error','error')
                    ncs1 =[];
                    return
                 end
            end
    end
    %-------------------------------------------------------------------    
    %check transmission interval conditions
    try
        hmin = evalin('base',char(get(handles.hmin,'String')));
    catch
        msgbox('Invalid value for minimum transmission interval entered','NCS Toolbox Error','error')
        ncs1 =[];
        return
    end
    
    if isnumeric(hmin) == 0 || numel(hmin)~=1 || hmin <= 0
         msgbox('The minimum transmission interval must be a strictly positive scalar of type double','NCS Toolbox Error','error')
         ncs1 =[];
         return
    end
    
    try
        hmax = evalin('base',char(get(handles.hmax,'String')));
    catch
        msgbox('Invalid value for maximum transmission interval entered','NCS Toolbox Error','error')
        ncs1 =[];
        return
    end   
    
    if isnumeric(hmax) == 0 || numel(hmax)~=1 || hmax < 0
        msgbox('The maximum transmission interval must be a positive scalar of type double','NCS Toolbox Error','error')
        ncs1 =[];
        return
    end
   
    if hmin > hmax
       msgbox('The minimum transmission interval must be greater than or equal to the maximum transmission interval','NCS Toolbox Error','error')
       ncs1 =[];
       return
    end
    
    %-------------------------------------------------------------------    
    %check delay interval conditions
    
    try
        taumin = evalin('base',char(get(handles.taumin,'String')));
    catch
        msgbox('Invalid value for minimum delay entered','NCS Toolbox Error','error')
        ncs1 =[];
        return
    end   

    if isnumeric(taumin) == 0 || numel(taumin)~=1 || taumin < 0
         msgbox('The minimum delay must be a positive scalar of type double','NCS Toolbox Error','error')
         ncs1 =[];
         return
    end
    
    try
        taumax = evalin('base',char(get(handles.taumax,'String')));
    catch
        msgbox('Invalid value for maximum delay entered','NCS Toolbox Error','error')
        ncs1 =[];
        return
    end   
    
    if isnumeric(taumax) == 0 || numel(taumax)~=1 || taumax < 0
        msgbox('The maximum delay must be a positive scalar of type double','NCS Toolbox Error','error')
        ncs1 =[];
        return
    end    
    
   
    if taumin > taumax
       msgbox('The minimum delay must be greater than or equal to the maximum delay','NCS Toolbox Error','error')
       ncs1 =[];
       return
    end    
     
    
    %-------------------------------------------------------------------    
    %check dropout conditions
            
    try
        delta = evalin('base',char(get(handles.delta,'String')));
    catch
        msgbox('Invalid value for max dropouts entered','NCS Toolbox Error','error')
        ncs1 =[];
        return
    end   
    
    if isnumeric(delta) == 0 || delta - round(delta) ~= 0 || delta < 0
        msgbox('Invalid delta parameter, must be a positive integer','NCS Toolbox Error','error')
        ncs1 =[];
        return
    end
    
    %-------------------------------------------------------------------    
    %Create Gamma matrics
    
    if get(handles.sharedCheckBox,'Value') 
        contents = get(handles.nodeList,'String');
    
        GammaU = cell(1,length(contents));
        GammaY = cell(1,length(contents));
        for ctr = 1:length(contents)
           GammaU{ctr} = zeros(size(B,2));
           GammaY{ctr} = zeros(size(C,1));

           if isempty(contents{ctr}(11:end))==0
                data=textscan(contents{ctr}(11:end),'%s','delimiter',',');

                for ctr2 =1:length(data{1})
                    spot = str2double(data{1}{ctr2}(2:end));
                    if strcmp(data{1}{ctr2}(1),'u')
                        GammaU{ctr}(spot,spot) = 1;
                    elseif strcmp(data{1}{ctr2}(1),'y')
                        GammaY{ctr}(spot,spot) = 1;
                    end
                end
           end

       end
   else
        GammaU{1} = eye(size(B,2));
        GammaY{1} = eye(size(C,1));
    end
   contents = get(handles.protocolChoice,'String');
   protocol = contents{get(handles.protocolChoice,'Value')};
   contents = get(handles.quantizer,'String');
   quantizer = contents{get(handles.quantizer,'Value')};
   
   %Create NCS object
    ncs1=ncs(size(B,2),size(C,1));
    ncs1 = ncs1.setPlant(A,B,C);
    switch controllerType
        case 'Static Feedback'
            ncs1 = ncs1.setController(K);
        case 'D-LTI  Observer-Based Feedback'
            ncs1 = ncs1.setController('D-LTI',Ac,Bc,Cc,K,L);
        case 'C-LTI  Dynamic Feedback'
            if uWired==0
                ncs1 = ncs1.setController('C-LTI',Ac,Bc,Cc,Dc);
            else
                ncs1 = ncs1.setController('C-LTI uWired',Ac,Bc,Cc,Dc);
            end
        case 'D-LTI  Dynamic Feedback'
            ncs1 = ncs1.setController('D-LTI',Ac,Bc,Cc,Dc);
    end

    temp=get(handles.refineButton,'UserData');
    tPoly=temp{1};
    timingDist=temp{2};
    
    if isempty(tPoly)
        % timing uncertainty editor wasn't used
        % genreate polygon from min-max info
        tPoly = genTimingUncertaintyRegion(hmin,hmax,taumin,taumax);
    end   
    
    network.tPoly       = tPoly;
    network.timingDist  = timingDist;
    network.maxDrops    = delta;
    network.nodesU      = GammaU;
    network.nodesY      = GammaY;
    network.protocol    = protocol;
    network.quantizer   = quantizer;
    
    ncs1 = ncs1.setNetwork(network);
    
% --- Executes during object creation, after setting all properties.
function hmin_CreateFcn(hObject, eventdata, handles)
% hObject    handle to hmin (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes during object creation, after setting all properties.
function taumax_CreateFcn(hObject, eventdata, handles)
% hObject    handle to taumax (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes during object creation, after setting all properties.
function taumin_CreateFcn(hObject, eventdata, handles)
% hObject    handle to taumin (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



% --- Executes during object creation, after setting all properties.
function hmax_CreateFcn(hObject, eventdata, handles)
% hObject    handle to hmax (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



% --- Executes during object creation, after setting all properties.
function edit6_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit6 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --------------------------------------------------------------------
function New_Callback(hObject, eventdata, handles)
% hObject    handle to New (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --------------------------------------------------------------------
function Help_Callback(hObject, eventdata, handles)
% hObject    handle to Help (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% --------------------------------------------------------------------
function About_Callback(hObject, eventdata, handles)
% hObject    handle to About (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

msgbox({'ncsToolbox was developed by:'  ...
        ' '...
        'Hybrid and Networked Systems Group' ...
        'University of Technology Eindhoven' ...
        ' ' ...
        'Questions or Comments can be sent to n.w.bauer@tue.nl'})

% --- Executes during object creation, after setting all properties.
function A_CreateFcn(hObject, eventdata, handles)
% hObject    handle to A (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes during object creation, after setting all properties.
function B_CreateFcn(hObject, eventdata, handles)
% hObject    handle to B (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes during object creation, after setting all properties.
function Ac_CreateFcn(hObject, eventdata, handles)
% hObject    handle to Ac (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes during object creation, after setting all properties.
function controllerType_CreateFcn(hObject, eventdata, handles)
% hObject    handle to controllerType (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes during object creation, after setting all properties.
function axes3_CreateFcn(hObject, eventdata, handles)
% hObject    handle to axes3 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: place code in OpeningFcn to populate axes3
    axes(hObject) %#ok<MAXES>
    Generate_NCS_Blockdiagram(hObject)


% --- Executes during object creation, after setting all properties.
function edit10_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit10 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes during object creation, after setting all properties.
function edit11_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit11 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes during object creation, after setting all properties.
function protocolChoice_CreateFcn(hObject, eventdata, handles)
% hObject    handle to protocolChoice (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes during object creation, after setting all properties.
function edit12_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit12 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes during object creation, after setting all properties.
function edit13_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit13 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes during object creation, after setting all properties.
function edit14_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit14 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
    if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
        set(hObject,'BackgroundColor','white');
    end

% --- Executes during object creation, after setting all properties.
function edit15_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit15 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
    if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
        set(hObject,'BackgroundColor','white');
    end


% --- Executes during object creation, after setting all properties.
function edit16_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit16 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
    if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
        set(hObject,'BackgroundColor','white');
    end

% --- Executes during object creation, after setting all properties.
function quantizer_CreateFcn(hObject, eventdata, handles)
% hObject    handle to quantizer (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
    if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
        set(hObject,'BackgroundColor','white');
    end


% --- Executes during object creation, after setting all properties.
function ncsGui_CreateFcn(hObject, eventdata, handles)
% hObject    handle to ncsEditor (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called


function A_Callback(hObject, eventdata, handles)
% hObject    handle to A (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of A as text
%        str2double(get(hObject,'String')) returns contents of A as a double


% --- Executes during object creation, after setting all properties.
function edit17_CreateFcn(hObject, eventdata, handles)
% hObject    handle to A (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
    if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
        set(hObject,'BackgroundColor','white');
    end



function B_Callback(hObject, eventdata, handles)
% hObject    handle to B (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of B as text
%        str2double(get(hObject,'String')) returns contents of B as a double


% --- Executes during object creation, after setting all properties.
function edit18_CreateFcn(hObject, eventdata, handles)
% hObject    handle to B (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
    if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
        set(hObject,'BackgroundColor','white');
    end



function C_Callback(hObject, eventdata, handles)
% hObject    handle to C (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of C as text
%        str2double(get(hObject,'String')) returns contents of C as a double


% --- Executes during object creation, after setting all properties.
function C_CreateFcn(hObject, eventdata, handles)
% hObject    handle to C (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
    if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
        set(hObject,'BackgroundColor','white');
    end



function Bc_Callback(hObject, eventdata, handles)
% hObject    handle to Bc (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of Bc as text
%        str2double(get(hObject,'String')) returns contents of Bc as a double


% --- Executes during object creation, after setting all properties.
function Bc_CreateFcn(hObject, eventdata, handles)
% hObject    handle to Bc (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
    if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
        set(hObject,'BackgroundColor','white');
    end



function Cc_Callback(hObject, eventdata, handles)
% hObject    handle to Cc (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of Cc as text
%        str2double(get(hObject,'String')) returns contents of Cc as a double


% --- Executes during object creation, after setting all properties.
function Cc_CreateFcn(hObject, eventdata, handles)
% hObject    handle to Cc (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
    if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
        set(hObject,'BackgroundColor','white');
    end



function Dc_Callback(hObject, eventdata, handles)
% hObject    handle to Dc (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of Dc as text
%        str2double(get(hObject,'String')) returns contents of Dc as a double


% --- Executes during object creation, after setting all properties.
function Dc_CreateFcn(hObject, eventdata, handles)
% hObject    handle to Dc (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
    if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
        set(hObject,'BackgroundColor','white');
    end


% --- Executes on selection change in controllerType.
function controllerType_Callback(hObject, eventdata, handles)
% hObject    handle to controllerType (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = cellstr(get(hObject,'String')) returns controllerType contents as cell array
%        contents{get(hObject,'Value')} returns selected item from controllerType


    contents = cellstr(get(hObject,'String'));
    selection = contents{get(hObject,'Value')};

    contents = cellstr(get(handles.ctlNodeSelect,'String'));
    activeNode = contents{get(handles.ctlNodeSelect,'Value')};
    
    nodeDependent = get(handles.nodeDepCheckBox,'Value');

    set(handles.AcText,'Visible','off')
    set(handles.BcText,'Visible','off')
    set(handles.CcText,'Visible','off')
    set(handles.DcText,'Visible','off')
    set(handles.EcText,'Visible','off')

    set(handles.Ac,'Visible','off')
    set(handles.Bc,'Visible','off')
    set(handles.Cc,'Visible','off')
    set(handles.Dc,'Visible','off')
    set(handles.Ec,'Visible','off')

    switch selection
        case 'Static Feedback'
            set(handles.uWiredCheckBox,'Enable','off');
            set(handles.uWiredCheckBox,'Value',0);
            if nodeDependent == 1
                set(handles.AcText,'String',['K' activeNode ':'])
            else
                set(handles.AcText,'String','K:')
            end

            set(handles.AcText,'Visible','on')
            set(handles.Ac,'Visible','on')

        case 'D-LTI  Observer-Based Feedback'
            set(handles.uWiredCheckBox,'Enable','off');
            set(handles.uWiredCheckBox,'Value',0);
            
            if nodeDependent == 1
                set(handles.AcText,'String',['A' activeNode ':'])
                set(handles.BcText,'String',['B' activeNode ':'])
                set(handles.CcText,'String',['C' activeNode ':'])
                set(handles.DcText,'String',['K' activeNode ':'])
                set(handles.EcText,'String',['L' activeNode ':'])           
            else
                set(handles.AcText,'String','A:')
                set(handles.BcText,'String','B:')
                set(handles.CcText,'String','C:')
                set(handles.DcText,'String','K:')
                set(handles.EcText,'String','L:')
            end

            set(handles.AcText,'Visible','on')
            set(handles.BcText,'Visible','on')
            set(handles.CcText,'Visible','on')
            set(handles.DcText,'Visible','on')
            set(handles.EcText,'Visible','on')

            set(handles.Ac,'Visible','on')
            set(handles.Bc,'Visible','on')
            set(handles.Cc,'Visible','on')
            set(handles.Dc,'Visible','on')
            set(handles.Ec,'Visible','on')

        case {'C-LTI  Dynamic Feedback'} 
            set(handles.uWiredCheckBox,'Enable','on');
            set(handles.AcText,'String','A:')
            set(handles.BcText,'String','B:')
            set(handles.CcText,'String','C:')
            set(handles.DcText,'String','D:')

            set(handles.AcText,'Visible','on')
            set(handles.BcText,'Visible','on')
            set(handles.CcText,'Visible','on')
            set(handles.DcText,'Visible','on')
            set(handles.EcText,'Visible','off')

            set(handles.Ac,'Visible','on')
            set(handles.Bc,'Visible','on')
            set(handles.Cc,'Visible','on')
            set(handles.Dc,'Visible','on')
            set(handles.Ec,'Visible','off')
            
            
        case{'D-LTI  Dynamic Feedback'}
            set(handles.uWiredCheckBox,'Enable','off');
            set(handles.uWiredCheckBox,'Value',0);
            
            if nodeDependent == 1
                set(handles.AcText,'String',['A' activeNode ':'])
                set(handles.BcText,'String',['B' activeNode ':'])
                set(handles.CcText,'String',['C' activeNode ':'])
                set(handles.DcText,'String',['D' activeNode ':'])            
            else
                set(handles.AcText,'String','A:')
                set(handles.BcText,'String','B:')
                set(handles.CcText,'String','C:')
                set(handles.DcText,'String','D:')
            end

            set(handles.AcText,'Visible','on')
            set(handles.BcText,'Visible','on')
            set(handles.CcText,'Visible','on')
            set(handles.DcText,'Visible','on')
            set(handles.EcText,'Visible','off')

            set(handles.Ac,'Visible','on')
            set(handles.Bc,'Visible','on')
            set(handles.Cc,'Visible','on')
            set(handles.Dc,'Visible','on')
            set(handles.Ec,'Visible','off')
    end
        
% --- Executes during object creation, after setting all properties.
function controlMode_CreateFcn(hObject, eventdata, handles)
% hObject    handle to controlMode (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes during object deletion, before destroying properties.
function uipanel3_DeleteFcn(hObject, eventdata, handles)
% hObject    handle to uipanel3 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --- Executes during object creation, after setting all properties.
function uipanel1_CreateFcn(hObject, eventdata, handles)
% hObject    handle to uipanel1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called


% --- Executes during object creation, after setting all properties.
function uipanel2_CreateFcn(hObject, eventdata, handles)
% hObject    handle to uipanel2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called


% --- Executes during object creation, after setting all properties.
function uipanel3_CreateFcn(hObject, eventdata, handles)
% hObject    handle to uipanel3 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called


% --- Executes during object creation, after setting all properties.
function uipanel4_CreateFcn(hObject, eventdata, handles)
% hObject    handle to uipanel4 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called


% --- Executes during object deletion, before destroying properties.
function ncsGui_DeleteFcn(hObject, eventdata, handles)
% hObject    handle to ncsEditor (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --- Executes when user attempts to close ncsEditor.
function ncsGui_CloseRequestFcn(hObject, eventdata, handles)
% hObject    handle to ncsEditor (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: delete(hObject) closes the figure
button = questdlg('Are you sure you want to quit?  All un-exported data will be lost.','NCS Editor','Ok','Cancel','Cancel'); 
if strcmp(button,'Ok')
    delete(hObject);
end

% --- Executes during object deletion, before destroying properties.
function uipanel1_DeleteFcn(hObject, eventdata, handles)
% hObject    handle to uipanel1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --- Executes during object deletion, before destroying properties.
function uipanel2_DeleteFcn(hObject, eventdata, handles)
% hObject    handle to uipanel2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --- Executes during object deletion, before destroying properties.
function uipanel4_DeleteFcn(hObject, eventdata, handles)
% hObject    handle to uipanel4 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --- Executes during object deletion, before destroying properties.
function axes3_DeleteFcn(hObject, eventdata, handles)
% hObject    handle to axes3 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --- Executes on selection change in ovraprx.
function ovraprx_Callback(hObject, eventdata, handles)
% hObject    handle to ovraprx (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = get(hObject,'String') returns ovraprx contents as cell array
%        contents{get(hObject,'Value')} returns selected item from ovraprx


% --- Executes on button press in pushbutton4.
function pushbutton4_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton4 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
    try
        B = evalin('base',char(get(handles.B,'String')));
    catch
        msgbox('Must first enter a valid value for input field B','NCS Toolbox Error','error')
        return
    end
    if isnumeric(B) == 0
        msgbox('Input field B must be a matrix of type double','NCS Toolbox Error','error')
       return
    end
    
    try
        C = evalin('base',char(get(handles.C,'String')));
    catch
        msgbox('Must first enter a valid value for input field C','NCS Toolbox Error','error')
        return
    end
    if isnumeric(C) == 0
        msgbox('Input field C must be a matrix of type double','NCS Toolbox Error','error')
       return
    end 

    %pass the handle of the ncsGui to the nodeGui
    nodes(handles.ncsGui);

% Old Code ------------
% load([savePath.path '\temp']);
% delete([savePath.path '\temp.mat'])
% if isempty(newNodeList) == 0
%     set(handles.nodeList,'String',newNodeList);
% end
%load('nodeGui_handles','nodeGui_handles');
% Old Code ------------

% --- Executes on selection change in nodeList.
function nodeList_Callback(hObject, eventdata, handles)
% hObject    handle to nodeList (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = cellstr(get(hObject,'String')) returns nodeList contents as cell array
%        contents{get(hObject,'Value')} returns selected item from nodeList


% --- Executes during object creation, after setting all properties.
function nodeList_CreateFcn(hObject, eventdata, handles)
% hObject    handle to nodeList (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: listbox controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
    if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
        set(hObject,'BackgroundColor','white');
    end
    set(hObject,'FontName','FixedWidth');


% --- Executes on selection change in protocolChoice.
function protocolChoice_Callback(hObject, eventdata, handles)
% hObject    handle to protocolChoice (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = cellstr(get(hObject,'String')) returns protocolChoice contents as cell array
%        contents{get(hObject,'Value')} returns selected item from protocolChoice


% --- Executes on button press in sharedCheckBox.
function sharedCheckBox_Callback(hObject, eventdata, handles)
% hObject    handle to sharedCheckBox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of sharedCheckBox

    if get(hObject,'Value') == 0
        set(handles.nodeList,'Enable','off');
        %set(handles.nodeList,'String','');
        set(handles.ctlNodeSelect,'Value',1);
        %set(handles.ctlNodeSelect,'String','1');
        ctlNodeSelect_Callback(handles.ctlNodeSelect, eventdata, handles);
        set(handles.ctlNodeSelect,'Enable','off')
        set(handles.pushbutton4,'Enable','off');
        set(handles.protocolChoice,'Enable','off');
        set(handles.nodeDepCheckBox,'Value',0);

        contents = cellstr(get(handles.controllerType,'String'));
        selection = contents{get(handles.controllerType,'Value')};
        switch selection
            case 'Static Feedback'
                set(handles.AcText,'String','K:')
            case 'D-LTI  Observer-Based Feedback'
                set(handles.AcText,'String','A:')
                set(handles.BcText,'String','B:')
                set(handles.CcText,'String','C:')
                set(handles.DcText,'String','K:')
                set(handles.EcText,'String','L:')
            case {'C-LTI  Dynamic Feedback', 'D-LTI  Dynamic Feedback'}
                set(handles.AcText,'String','A:')
                set(handles.BcText,'String','B:')
                set(handles.CcText,'String','C:')
                set(handles.DcText,'String','D:')
        end    

    else
        set(handles.nodeList,'Enable','on');
        if isempty(get(handles.nodeList,'String'))
            set(handles.nodeList,'String',{'node1     '});
        end
        set(handles.nodeList,'Value',1);
        set(handles.pushbutton4,'Enable','on');
        set(handles.protocolChoice,'Enable','on');   
    end



function Ac_Callback(hObject, eventdata, handles)
% hObject    handle to Ac (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of Ac as text
%        str2double(get(hObject,'String')) returns contents of Ac as a double


% --- Executes on key press with focus on hmax and none of its controls.
function hmax_KeyPressFcn(hObject, eventdata, handles)
% hObject    handle to hmax (see GCBO)
% eventdata  structure with the following fields (see UICONTROL)
%	Key: name of the key that was pressed, in lower case
%	Character: character interpretation of the key(s) that was pressed
%	Modifier: name(s) of the modifier key(s) (i.e., control, shift) pressed
% handles    structure with handles and user data (see GUIDATA)
set(hObject,'ForegroundColor','black')

% --- Executes on key press with focus on taumin and none of its controls.
function taumin_KeyPressFcn(hObject, eventdata, handles)
% hObject    handle to taumin (see GCBO)
% eventdata  structure with the following fields (see UICONTROL)
%	Key: name of the key that was pressed, in lower case
%	Character: character interpretation of the key(s) that was pressed
%	Modifier: name(s) of the modifier key(s) (i.e., control, shift) pressed
% handles    structure with handles and user data (see GUIDATA)
set(hObject,'ForegroundColor','black')


% --- Executes on key press with focus on hmin and none of its controls.
function hmin_KeyPressFcn(hObject, eventdata, handles)
% hObject    handle to hmin (see GCBO)
% eventdata  structure with the following fields (see UICONTROL)
%	Key: name of the key that was pressed, in lower case
%	Character: character interpretation of the key(s) that was pressed
%	Modifier: name(s) of the modifier key(s) (i.e., control, shift) pressed
% handles    structure with handles and user data (see GUIDATA)
set(hObject,'ForegroundColor','black')


% --- Executes on key press with focus on taumax and none of its controls.
function taumax_KeyPressFcn(hObject, eventdata, handles)
% hObject    handle to taumax (see GCBO)
% eventdata  structure with the following fields (see UICONTROL)
%	Key: name of the key that was pressed, in lower case
%	Character: character interpretation of the key(s) that was pressed
%	Modifier: name(s) of the modifier key(s) (i.e., control, shift) pressed
% handles    structure with handles and user data (see GUIDATA)
set(hObject,'ForegroundColor','black')


% --- Executes on key press with focus on delta and none of its controls.
function delta_KeyPressFcn(hObject, eventdata, handles)
% hObject    handle to delta (see GCBO)
% eventdata  structure with the following fields (see UICONTROL)
%	Key: name of the key that was pressed, in lower case
%	Character: character interpretation of the key(s) that was pressed
%	Modifier: name(s) of the modifier key(s) (i.e., control, shift) pressed
% handles    structure with handles and user data (see GUIDATA)
set(hObject,'ForegroundColor','black')

function hmin_Callback(hObject, eventdata, handles)
% hObject    handle to hmin (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of hmin as text
%        str2double(get(hObject,'String')) returns contents of hmin as a double

bounds(1) = str2double(get(handles.hmin,'String'));
bounds(2) = str2double(get(handles.hmax,'String'));
bounds(3) = str2double(get(handles.taumin,'String'));
bounds(4) = str2double(get(handles.taumax,'String'));

error=checkTimingBounds(bounds);

if error
   set(hObject,'String','');
   msgbox(error,'NCS Toolbox Error','error');
end

if ~error && ~isnan(bounds(1)) && ~isnan(bounds(2)) && ~isnan(bounds(3)) && ~isnan(bounds(4))
    set(handles.refineButton,'Enable','on');
else
    set(handles.refineButton,'Enable','off');
end 


function hmax_Callback(hObject, eventdata, handles)
% hObject    handle to hmax (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of hmax as text
%        str2double(get(hObject,'String')) returns contents of hmax as a double

bounds(1) = str2double(get(handles.hmin,'String'));
bounds(2) = str2double(get(handles.hmax,'String'));
bounds(3) = str2double(get(handles.taumin,'String'));
bounds(4) = str2double(get(handles.taumax,'String'));

error=checkTimingBounds(bounds);

if error
   set(hObject,'String','');
   msgbox(error,'NCS Toolbox Error','error');
end

if ~error && ~isnan(bounds(1)) && ~isnan(bounds(2)) && ~isnan(bounds(3)) && ~isnan(bounds(4))
    set(handles.refineButton,'Enable','on');
else
    set(handles.refineButton,'Enable','off');
end 

function taumin_Callback(hObject, eventdata, handles)
% hObject    handle to taumin (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of taumin as text
%        str2double(get(hObject,'String')) returns contents of taumin as a double

bounds(1) = str2double(get(handles.hmin,'String'));
bounds(2) = str2double(get(handles.hmax,'String'));
bounds(3) = str2double(get(handles.taumin,'String'));
bounds(4) = str2double(get(handles.taumax,'String'));

error=checkTimingBounds(bounds);

if error
   set(hObject,'String','');
   msgbox(error,'NCS Toolbox Error','error');
end

if ~error && ~isnan(bounds(1)) && ~isnan(bounds(2)) && ~isnan(bounds(3)) && ~isnan(bounds(4))
    set(handles.refineButton,'Enable','on');
else
    set(handles.refineButton,'Enable','off');
end  

function taumax_Callback(hObject, eventdata, handles)
% hObject    handle to taumax (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of taumax as text
%        str2double(get(hObject,'String')) returns contents of taumax as a double

bounds(1) = str2double(get(handles.hmin,'String'));
bounds(2) = str2double(get(handles.hmax,'String'));
bounds(3) = str2double(get(handles.taumin,'String'));
bounds(4) = str2double(get(handles.taumax,'String'));

error=checkTimingBounds(bounds);

if error
   set(hObject,'String','');
   msgbox(error,'NCS Toolbox Error','error');
end

if ~error && ~isnan(bounds(1)) && ~isnan(bounds(2)) && ~isnan(bounds(3)) && ~isnan(bounds(4))
    set(handles.refineButton,'Enable','on');
else
    set(handles.refineButton,'Enable','off');
end 

% --- If Enable == 'on', executes on mouse press in 5 pixel border.
% --- Otherwise, executes on mouse press in 5 pixel border or over taumax.
function taumax_ButtonDownFcn(hObject, eventdata, handles)
% hObject    handle to taumax (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)



function delta_Callback(hObject, eventdata, handles)
% hObject    handle to delta (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of delta as text
%        str2double(get(hObject,'String')) returns contents of delta as a double


% --- Executes when selected object is changed in uipanel2.
function uipanel2_SelectionChangeFcn(hObject, eventdata, handles)
% hObject    handle to the selected object in uipanel2 
% eventdata  structure with the following fields (see UIBUTTONGROUP)
%	EventName: string 'SelectionChanged' (read only)
%	OldValue: handle of the previously selected object or empty if none was selected
%	NewValue: handle of the currently selected object
% handles    structure with handles and user data (see GUIDATA)


% --------------------------------------------------------------------
function uipanel1_ButtonDownFcn(hObject, eventdata, handles)
% hObject    handle to uipanel1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --------------------------------------------------------------------
function uipanel2_ButtonDownFcn(hObject, eventdata, handles)
% hObject    handle to uipanel2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --- Executes on button press in StabilityButton.
function StabilityButton_Callback(hObject, eventdata, handles)
% hObject    handle to StabilityButton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

    ncs = buildNcs(handles);

    if isobject(ncs) == 1
        %jFigPeer = get(handle(gcf),'JavaFrame');
        %jWindow = jFigPeer.fFigureClient.getWindow;
        %jWindow.setEnabled(false);  % or true
        set(handles.ncsGui,'Visible','off');
        ncsAnalyzer(handles.ncsGui,ncs);
    end


% --- Executes on button press in nodeDepCheckBox.
function nodeDepCheckBox_Callback(hObject, eventdata, handles)
% hObject    handle to nodeDepCheckBox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of nodeDepCheckBox

    if get(handles.sharedCheckBox,'Value') == 0
        msgbox('Must enable shared medium in Network Properties and define nodes to enable node dependent controller!','NCS Toolbox')
        set(hObject,'Value',0);
        return
    end

    set(handles.ctlNodeSelect,'Value',1);
    ctlNodeSelect_Callback(handles.ctlNodeSelect, eventdata, handles);
    contents = cellstr(get(handles.controllerType,'String'));
    ctlType = contents{get(handles.controllerType,'Value')};

    if get(hObject,'Value') == 0
        set(handles.ctlNodeSelect,'Enable','off');
        if strcmp(ctlType,'Static Feedback')
            set(handles.AcText,'String','K:');
        elseif strcmp(ctlType,'C-LTI  Dynamic Feedback') || strcmp(ctlType,'D-LTI  Dynamic Feedback')
            set(handles.AcText,'String','A:');
            set(handles.BcText,'String','B:');
            set(handles.CcText,'String','C:');
            set(handles.DcText,'String','D:');
        elseif strcmp(ctlType,'D-LTI  Observer-Based Feedback')
            set(handles.AcText,'String','A:');
            set(handles.BcText,'String','B:');
            set(handles.CcText,'String','C:');
            set(handles.DcText,'String','K:');
            set(handles.EcText,'String','L:');
        end
    else
        set(handles.ctlNodeSelect,'Enable','on');
        
        if strcmp(ctlType,'Static Feedback')
            set(handles.AcText,'String','K1:');
        elseif strcmp(ctlType,'C-LTI  Dynamic Feedback') || strcmp(ctlType,'D-LTI  Dynamic Feedback')
            set(handles.AcText,'String','A1:');
            set(handles.BcText,'String','B1:');
            set(handles.CcText,'String','C1:');
            set(handles.DcText,'String','D1:');
        elseif strcmp(ctlType,'D-LTI  Observer-Based Feedback')
            set(handles.AcText,'String','A1:');
            set(handles.BcText,'String','B1:');
            set(handles.CcText,'String','C1:');
            set(handles.DcText,'String','K1:');
            set(handles.EcText,'String','L1:');
        end

    end


% --- Executes on selection change in ctlNodeSelect.
function ctlNodeSelect_Callback(hObject, eventdata, handles)
% hObject    handle to ctlNodeSelect (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = cellstr(get(hObject,'String')) returns ctlNodeSelect contents as cell array
%        contents{get(hObject,'Value')} returns selected item from ctlNodeSelect
 
    contents = cellstr(get(handles.controllerType,'String'));
    ctlType = contents{get(handles.controllerType,'Value')};

    contents = cellstr(get(hObject,'String'));
    activeNode = str2double(contents{get(hObject,'Value')});
        
    lastNode = get(hObject,'UserData');   
    if isempty(lastNode)
        lastNode = 1;
    end
    
    
    if lastNode~=activeNode  %no need to change anything if node stays the same
        
        %Save Data
        AcData = get(handles.Ac,'UserData');            % retrive saved data
        AcData{lastNode} = get(handles.Ac,'String');    % store new input
        set(handles.Ac,'UserData',AcData)               % put saved into handles.Ac       

        BcData = get(handles.Bc,'UserData');            % retrive saved data
        BcData{lastNode} = get(handles.Bc,'String');    % store new input
        set(handles.Bc,'UserData',BcData)               % put saved into handles.Bc       

        CcData = get(handles.Cc,'UserData');            % retrive saved data
        CcData{lastNode} = get(handles.Cc,'String');    % store new input
        set(handles.Cc,'UserData',CcData)               % put saved into handles.Cc       

        DcData = get(handles.Dc,'UserData');            % retrive saved data
        DcData{lastNode} = get(handles.Dc,'String');    % store new input
        set(handles.Dc,'UserData',DcData)               % put saved into handles.Dc          

        EcData = get(handles.Ec,'UserData');            % retrive saved data
        EcData{lastNode} = get(handles.Ec,'String');    % store new input
        set(handles.Ec,'UserData',EcData)               % put saved into handles.Ec       

        % Input saved data
        if length(AcData) >= activeNode
            set(handles.Ac,'String',AcData{activeNode});    % insert current value into cell
            set(handles.Bc,'String',BcData{activeNode});    % insert current value into cell
            set(handles.Cc,'String',CcData{activeNode});    % insert current value into cell
            set(handles.Dc,'String',DcData{activeNode});    % insert current value into cell
            set(handles.Ec,'String',EcData{activeNode});    % insert current value into cell
        else
            set(handles.Ac,'String','');              
            set(handles.Bc,'String','');                 
            set(handles.Cc,'String','');                    
            set(handles.Dc,'String','');    
            set(handles.Ec,'String',''); 
        end   
        set(hObject,'UserData',activeNode);     % store last node in userData field
        
        %Change Text
        if strcmp(ctlType,'Static Feedback')
            set(handles.AcText,'String',['K' num2str(activeNode) ':']);
        elseif strcmp(ctlType,'C-LTI  Dynamic Feedback') || strcmp(ctlType,'D-LTI  Dynamic Feedback')
            set(handles.AcText,'String',['A' num2str(activeNode) ':']);
            set(handles.BcText,'String',['B' num2str(activeNode) ':']);
            set(handles.CcText,'String',['C' num2str(activeNode) ':']);
            set(handles.DcText,'String',['D' num2str(activeNode) ':']);

        elseif strcmp(ctlType,'D-LTI  Observer-Based Feedback')
            set(handles.AcText,'String',['A' num2str(activeNode) ':']);
            set(handles.BcText,'String',['B' num2str(activeNode) ':']);
            set(handles.CcText,'String',['C' num2str(activeNode) ':']);
            set(handles.DcText,'String',['K' num2str(activeNode) ':']);
            set(handles.EcText,'String',['L' num2str(activeNode) ':']);
        end
        
    end
    
    
% --- Executes during object creation, after setting all properties.
function ctlNodeSelect_CreateFcn(hObject, eventdata, handles)
% hObject    handle to ctlNodeSelect (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in uWiredCheckBox.
function uWiredCheckBox_Callback(hObject, eventdata, handles)
% hObject    handle to uWiredCheckBox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of uWiredCheckBox



function Ec_Callback(hObject, eventdata, handles)
% hObject    handle to Ec (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of Ec as text
%        str2double(get(hObject,'String')) returns contents of Ec as a double


% --- Executes during object creation, after setting all properties.
function Ec_CreateFcn(hObject, eventdata, handles)
% hObject    handle to Ec (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on selection change in quantizer.
function quantizer_Callback(hObject, eventdata, handles)
% hObject    handle to quantizer (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = cellstr(get(hObject,'String')) returns quantizer contents as cell array
%        contents{get(hObject,'Value')} returns selected item from quantizer


% --- Executes on button press in simulateButton.
function simulateButton_Callback(hObject, eventdata, handles)
% hObject    handle to simulateButton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
    ncs = buildNcs(handles);

    if isobject(ncs)
        %jFigPeer = get(handle(gcf),'JavaFrame');
        %jWindow = jFigPeer.fFigureClient.getWindow;
        %jWindow.setEnabled(false);  % or true
        set(handles.ncsGui,'Visible','off');
        h=ncsSimulator(handles.ncsGui,ncs);
        %set(h,'position',[0 0 30 40]) 
    end


% --- Executes on button press in refineButton.
function refineButton_Callback(hObject, eventdata, handles)
% hObject    handle to refineButton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
    
    %set(handles.ncsGui,'Visible','off');
    %pass the handle of the ncsGui to the nodeGui
    try
        hmin = evalin('base',char(get(handles.hmin,'String')));
    catch
        msgbox('Invalid value for minimum transmission interval entered','NCS Toolbox Error','error')
        ncs1 =[];
        return
    end
    
    if isnumeric(hmin) == 0 || numel(hmin)~=1 || hmin <= 0
         msgbox('The minimum transmission interval must be a strictly positive scalar of type double','NCS Toolbox Error','error')
         ncs1 =[];
         return
    end
    
    try
        hmax = evalin('base',char(get(handles.hmax,'String')));
    catch
        msgbox('Invalid value for maximum transmission interval entered','NCS Toolbox Error','error')
        ncs1 =[];
        return
    end   
    
    if isnumeric(hmax) == 0 || numel(hmax)~=1 || hmax < 0
        msgbox('The maximum transmission interval must be a positive scalar of type double','NCS Toolbox Error','error')
        ncs1 =[];
        return
    end
   
    if hmin > hmax
       msgbox('The minimum transmission interval must be greater than or equal to the maximum transmission interval','NCS Toolbox Error','error')
       ncs1 =[];
       return
    end

    if(~get(handles.ncsGui,'UserData'))
        set(handles.ncsGui,'UserData',1);
        ncsTimingUncertainty(handles.ncsGui);
    end

% --- If Enable == 'on', executes on mouse press in 5 pixel border.
% --- Otherwise, executes on mouse press in 5 pixel border or over taumin.
function taumin_ButtonDownFcn(hObject, eventdata, handles)
% hObject    handle to taumin (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
