function varargout = ncsTimingUncertainty(varargin)
% Copyright
% Copyright is with the following author(s):
%
% (C) 2012 Nick Bauer, Eindhoven University of Technology, The Netherlands, n.w.bauer@tue.nl
% (C) 2012 Bas van Loon, Eindhoven University of Technology, The Netherlands, s.j.l.m.v.Loon@tue.nl
% (C) 2012 Maurice Heemels, Eindhoven University of Technology, The Netherlands, m.heemels@tue.nl
% -------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------
% Edit the above text to modify the response to help ncsTimingUncertainty

% Last Modified by GUIDE v2.5 06-Dec-2013 13:23:27

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @ncsTimingUncertainty_OpeningFcn, ...
                   'gui_OutputFcn',  @ncsTimingUncertainty_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT
    

function [h x_vert y_vert]=drawPoly(contents)
    x_vert=zeros(length(contents),1);
    y_vert=zeros(length(contents),1);
    for ctr=1:length(contents)
        [x_str rest] =strtok(contents{ctr},',');
        y_str = rest(3:end);
        x_vert(ctr)=str2double(x_str);
        y_vert(ctr)=str2double(y_str);
    end
    if length(contents)>=3
        if (polyarea(x_vert,y_vert))    %make sure points have a positive area before using convhull
            k=convhull(x_vert,y_vert);
        else
            k=1:length(x_vert);
        end
        h=patch(x_vert(k),y_vert(k),'r','EdgeColor','r');
    elseif length(contents)>=1
        h=patch(x_vert,y_vert,'r','EdgeColor','r');
    else
        h=[];
        x_vert=[];
        y_vert=[];
    end

function updateFigure(handles,data,action)
    
    %get current handles
    h=get(handles.axes1,'UserData');
    unRefinedPoly   = h{1};
    refinedPoly     = h{2};
    dataPlot        = h{3};
    activePoint     = h{4};
    refinedVerts    = h{5};
    includedMeasDataPts = h{6};
    timingDist          = h{7};
    
    %----------------------------------------------------------
    %update points which generate plot accordingly
    
    if strcmp(action,'unRefined')
        %data is a complete set of points
   
        if ~isempty(unRefinedPoly)
            delete(unRefinedPoly);
        end
        
        unRefinedPoly=patch(data(:,1),data(:,2),'r','FaceColor','none');
        set(unRefinedPoly,'EdgeColor','b','LineStyle',':')
    elseif strcmp(action,'refined_new')
        %data is a complete set of points
        
        if ~isempty(refinedPoly)
            delete(refinedPoly)
        end
        
        if isempty(data)
            %nothing to do here, clear data
            refinedPoly=[];
            return;
        end
        
        %draw poly
        if (polyarea(data(:,1),data(:,2)))  %make sure points have a positive area before using convhull
            k=convhull(data(:,1),data(:,2));
        else
            k=1:size(data,1);
        end
        refinedPoly=patch(data(k,1),data(k,2),'r','EdgeColor','r');
        
        %plot new polygon verts
        if ~isempty(refinedVerts)
            delete(refinedVerts);
        end
        refinedVerts=plot(data(:,1),data(:,2),'g+');
        
        %update stored vertList
        contents = cell(0);
        for ctr=1:size(data,1)
            contents{ctr}=[num2str(data(ctr,1)) ', ' num2str(data(ctr,2))];
        end
        set(handles.vertList, 'String', contents);
        set(handles.vertList,'Value',1);
        set(handles.vertList, 'UserData', data);
        
    elseif strcmp(action,'refined_add')
        %data is a single point
        
        %add point to vert list
        contents = get(handles.vertList,'String');
        contents{end+1} = [num2str(data(1)) ', ' num2str(data(2))];

        set(handles.vertList,'String',contents);
        set(handles.vertList,'Value',length(contents));
        
        %add point to saved vert list
        storedVerts = get(handles.vertList,'UserData');
        set(handles.vertList,'UserData',[storedVerts;data(1) data(2)]);
        
        %plot new polygon
        if ~isempty(refinedPoly)
            delete(refinedPoly);
        end
        [refinedPoly x_verts y_verts]=drawPoly(contents);
        
        %plot new polygon verts
        if ~isempty(refinedVerts)
            delete(refinedVerts);
        end
        refinedVerts=plot(x_verts,y_verts,'g+');
       
    elseif strcmp(action,'refined_delete')
        contents = get(handles.vertList,'String');
        ptToBeDeleted = get(handles.vertList,'Value');

        %update stored vertList
        contents{ptToBeDeleted}= '';
        contents(cellfun(@isempty,contents)) = '';

        for ctr = ptToBeDeleted:length(contents)
            contents{ctr} = contents{ctr};
        end
        
        if ptToBeDeleted > length(contents)
            set(handles.vertList,'Value',length(contents))
        end
        set(handles.vertList,'String',contents);

        %plot new polygon
        if ~isempty(refinedPoly)
            delete(refinedPoly);
        end
        [refinedPoly x_verts y_verts]=drawPoly(contents);
        set(handles.vertList,'UserData',[x_verts y_verts]);
        
        %plot new polygon verts
        if ~isempty(refinedVerts)
            delete(refinedVerts);
            refinedVerts=[];
        end
        if ~isempty(refinedPoly)
            refinedVerts=plot(x_verts,y_verts,'g+');
        end        
        
    elseif strcmp(action,'plot_data')
        %data is a list of measured data points
        if ~isempty(dataPlot)
            delete(dataPlot)
        end
        
        if(~isempty(data))
            res=str2double(get(handles.measResBox,'String')); 
            [pt timingDist] = genUncertainPolyFromMeasurementData(data,0,res);

            for i=1:length(pt.sq)
                dataPlot(i) = patch(pt.sq{i}(:,1),pt.sq{i}(:,2),'r');
                set(dataPlot(i),'FaceColor','flat','FaceVertexCData',pt.color{i},'CDataMapping','scaled');
            end     

            % adjust limits so measurements are in view
            ptMax=max(max([pt.sq{:}]));
            xlim = get(handles.axes1, 'XLim');
            set(handles.axes1, 'XLim', [0 max(xlim(2),ptMax*1.2)]);
            ylim = get(handles.axes1, 'YLim');
            set(handles.axes1, 'YLim', [0 max(ylim(2),ptMax*1.2)]);
            %draw small-delay bound line
            plot([0 1.2*ptMax],[0 1.2*ptMax],'k--');

            colorbar();
            colormap autumn

            dataPlot = dataPlot(end:-1:1);  %reverse order
        else
            dataPlot=[];
            timingDist=[];
        end
    end
    
    %----------------------------------------------------------
    %circle the new active point
    if ~isempty(activePoint)
        delete(activePoint);
        activePoint=[];
    end
    
    vertList = get(handles.vertList,'String');
    if ~isempty(vertList)
        numSelected = get(handles.vertList,'Value');
        onVert=vertList{numSelected};
        
        [x_str rest] =strtok(onVert,',');
        y_str = rest(3:end);
        x_vert=str2double(x_str);
        y_vert=str2double(y_str);

        activePoint=plot(x_vert,y_vert,'go');
    end
    
    %----------------------------------------------------------
    %reshuffle plots to display in the 'right' order...
    

    
    if ~isempty(refinedPoly)
        uistack(refinedPoly,'top');
    end
    
     uistack(unRefinedPoly,'top');   
    
    if ~isempty(dataPlot)
        for ctr=1:length(dataPlot)
            uistack(dataPlot(ctr),'top');
            %set(dataPlot(ctr),'FaceColor','flat','FaceVertexCData',50,'CDataMapping','scaled')
        end
    end
    if ~isempty(refinedVerts)
        uistack(refinedVerts,'top');
    end
    if ~isempty(activePoint)
        uistack(activePoint,'top');
    end

   
    
    drawnow;
    
    %----------------------------------------------------------
    %store new handles
    set(handles.axes1,'UserData',{unRefinedPoly, refinedPoly, dataPlot, activePoint, refinedVerts, includedMeasDataPts,timingDist});

% --- Executes just before ncsTimingUncertainty is made visible.
function ncsTimingUncertainty_OpeningFcn(hObject, eventdata, handles, varargin) %#ok<*INUSL>
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to ncsTimingUncertainty (see VARARGIN)

% Choose default command line output for ncsTimingUncertainty
% handles.output = hObject;

%mainGuiInput = find(strcmp(varargin, 'ncsEditor'));
% Remember the handle, and adjust our position
    handles.ncsEditor = varargin{1};
    
    % Obtain handles using GUIDATA with the caller's handle 
    mainHandles = guidata(handles.ncsEditor);    
    set(handles.axes1,'UserData',{[] [] [] [] [] [] []});
    

        
    %draw unRefined region
    hmin=str2double(get(mainHandles.hmin,'String'));
    hmax=str2double(get(mainHandles.hmax,'String'));
    taumin=str2double(get(mainHandles.taumin,'String'));
    taumax=str2double(get(mainHandles.taumax,'String'));
    
    set(handles.hmin,'String',hmin);
    set(handles.hmax,'String',hmax);
    set(handles.taumin,'String',taumin);
    set(handles.taumax,'String',taumax);
    
    verts=genTimingUncertaintyRegion(hmin,hmax,taumin,taumax);
    updateFigure(handles,verts,'unRefined');
    hold all;
    handles.uncertainBounds=verts;

    %draw small-delay bound line
    plot([0 100],[0 100],'k--');    
    %set xlim ylim
    m=max(max(verts));
    set(handles.axes1,'XLim',[0 m*1.2])
    set(handles.axes1,'YLim',[0 m*1.2])
    
    %draw refined region, if exists
    temp = get(mainHandles.refineButton,'UserData');
    if ~isempty(temp{1})
        tPoly=temp{1};
        updateFigure(handles,tPoly,'refined_new');
    else
        minMaxFitButton_Callback(hObject, eventdata, handles)
    end
    
    
    legend('Small-delay boundry','Timing Uncertainty Polygon','Min-Max Shape')
    
    %draw measurement data, if exists
    colorbar
    if ~isempty(temp{2})
        set(handles.probabilityLabel,'Visible','on');
        measData= temp{2};
        updateFigure(handles,measData,'plot_data');
        set(handles.genPolyButton,'Enable','on');
        set(handles.thresholdBox,'Enable','on');
    else
        colorbar('hide')
        set(handles.clearButton,'Enable','off');
    end    
    
    
    
    %------------------------------
    % Set the text equal to the main GUI's text box

    % Update handles structure
    guidata(hObject, handles);
    % UIWAIT makes ncsTimingUncertainty wait for user response (see UIRESUME)
    %uiwait(handles.timingGui);


% --- Outputs from this function are returned to the command line.
function varargout = ncsTimingUncertainty_OutputFcn(hObject, eventdata, handles)  %#ok<*INUSD>
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = [];


% --- Executes on button press in OkButton.
function OkButton_Callback(hObject, eventdata, handles)
% hObject    handle to OkButton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
    
    
    vertList=get(handles.vertList,'UserData');
    temp=get(handles.axes1,'UserData');         %retrieve currently plotted data
    timingDist=temp{7};
    if ~isempty(vertList)
        if(any(vertList(:,2)>vertList(:,1)))
            msgbox('Some vertices lie outside of the small-delay range. Adjust these points before saving.','NCS Toolbox Error','error');
            return;
        end
    end
    
    main = handles.ncsEditor;
    mainHandles = guidata(main);
    
    if(~isempty(vertList))
       hmin = min(vertList(:,1));
       hmax = max(vertList(:,1));
       taumin = min(vertList(:,2));
       taumax = max(vertList(:,2));
    else
       hmin = '';
       hmax = '';
       taumin = '';
       taumax = '';        
    end
    
    set(mainHandles.refineButton,'UserData',{vertList,timingDist});
    set(mainHandles.hmin,'String',hmin);
    set(mainHandles.hmax,'String',hmax);
    set(mainHandles.taumin,'String',taumin);
    set(mainHandles.taumax,'String',taumax);
    
    if(~isempty(vertList))
        %disable user from changing boxes, values are now based on polygon
        set(mainHandles.hmin,'Enable','Off');
        set(mainHandles.hmax,'Enable','Off');
        set(mainHandles.taumin,'Enable','Off');
        set(mainHandles.taumax,'Enable','Off');
    else
        %no polygon, so enable user to modify values
        set(mainHandles.hmin,'Enable','On');
        set(mainHandles.hmax,'Enable','On');
        set(mainHandles.taumin,'Enable','On');
        set(mainHandles.taumax,'Enable','On');       
    end
    timingGui_CloseRequestFcn(hObject, eventdata, handles);

    

% --- Executes during object creation, after setting all properties.
function timingGui_CreateFcn(hObject, eventdata, handles)
% hObject    handle to timingGui (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called


% --- Executes on selection change in vertList.
function vertList_Callback(hObject, eventdata, handles)
% hObject    handle to vertList (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = cellstr(get(hObject,'String')) returns vertList contents as cell array
%        contents{get(hObject,'Value')} returns selected item from vertList

set(hObject,'Enable','off');
drawnow;
updateFigure(handles,[],'');
set(hObject,'Enable','on');
uicontrol(hObject);
    
% --- Executes during object creation, after setting all properties.
function vertList_CreateFcn(hObject, eventdata, handles)
% hObject    handle to vertList (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: listbox controls usually have a white background on Windows.
%       See ISPC and COMPUTER.

if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

set(hObject,'FontName','FixedWidth');

% --- Executes on button press in DeleteButton.
function DeleteButton_Callback(hObject, eventdata, handles)
% hObject    handle to DeleteButton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

    contents = get(handles.vertList,'String');
    
    if ~isempty(contents)
        set(hObject,'Enable','off');
        drawnow;
        updateFigure(handles,[],'refined_delete')
        set(hObject,'Enable','on');
    end
    
% --- Executes on button press in cancelButton.
function cancelButton_Callback(hObject, eventdata, handles)
% hObject    handle to cancelButton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

%uiresume(handles.timingGui);
timingGui_CloseRequestFcn(hObject, eventdata, handles);

% --- Executes when user attempts to close timingGui.
function timingGui_CloseRequestFcn(hObject, eventdata, handles)
% hObject    handle to timingGui (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

    %uiresume(hObject);
    main = handles.ncsEditor;
    mainHandles = guidata(main);
    set(mainHandles.ncsGui,'UserData',0);
    delete(handles.timingGui);
    
% --- Executes on mouse press over axes background.
function axes1_ButtonDownFcn(hObject, eventdata, handles)
% hObject    handle to axes1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)



% --- Executes on button press in Addbutton.
function Addbutton_Callback(hObject, eventdata, handles)
% hObject    handle to Addbutton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
    zoom off;
    pan off;
    set(handles.Addbutton,'Enable','off')
    set(handles.zoomButton,'Enable','off')
    set(handles.panButton,'Enable','off')
    set(handles.DeleteButton,'Enable','off')
    drawnow;

    set(handles.text5,'Visible','on');
    [x,y,button] = ginput(1);
    while button==1
        
        %if inpolygon(x,y,handles.uncertainBounds(:,1),handles.uncertainBounds(:,2))
        if (x>=y && x>=0 && y>=0)
            %take exact coordinates
            updateFigure(handles,[x y],'refined_add');
        end
        [x,y,button] = ginput(1);   %allow to add another point
    end
    set(handles.text5,'Visible','off');
    set(handles.zoomButton,'Enable','on')
    set(handles.panButton,'Enable','on')
    set(handles.DeleteButton,'Enable','on')
    set(handles.Addbutton,'Enable','on')

% --- Executes on button press in plotDataButton.
function plotDataButton_Callback(hObject, eventdata, handles)
% hObject    handle to plotDataButton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
    try
        data=evalin('base',get(handles.measurementDataBox,'String'));
    catch
       msgbox('Attempted import of measured data failed.  Data must be a two column matrix of points.','NCS Toolbox Error','error');
       return;
    end
    
    if(isempty(data))
       msgbox('Attempted import of measured data failed.  Data must be a two column matrix of points.','NCS Toolbox Error','error');
       return;        
    end
    
    try
        res=str2double(get(handles.measResBox,'String'));
    catch
        msgbox('Resolution must be a positive number.','NCS Toolbox Error','error')
        return;
    end

    if ~isnumeric(res) || isnan(res)
        msgbox('Resolution must be a positive number.','NCS Toolbox Error','error')
        return;
    end

    if(res<=0)
        msgbox('Resolution must be a positive number.','NCS Toolbox Error','error')
        return;
    end
    
    
    set(hObject,'Enable','off');
    drawnow;
    updateFigure(handles,data,'plot_data');
    set(handles.measurementDataBox,'UserData',data);
    set(handles.probabilityLabel,'Visible','on');
    set(handles.thresholdBox,'Enable','on');
    set(handles.genPolyButton,'Enable','on');
    set(handles.clearButton,'Enable','on');
    set(hObject,'Enable','on');

function measurementDataBox_Callback(hObject, eventdata, handles)
% hObject    handle to measurementDataBox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of measurementDataBox as text
%        str2double(get(hObject,'String')) returns contents of measurementDataBox as a double


% --- Executes during object creation, after setting all properties.
function measurementDataBox_CreateFcn(hObject, eventdata, handles)
% hObject    handle to measurementDataBox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function thresholdBox_Callback(hObject, eventdata, handles)
% hObject    handle to thresholdBox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of thresholdBox as text
%        str2double(get(hObject,'String')) returns contents of thresholdBox as a double


% --- Executes during object creation, after setting all properties.
function thresholdBox_CreateFcn(hObject, eventdata, handles)
% hObject    handle to thresholdBox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in zoomButton.
function zoomButton_Callback(hObject, eventdata, handles)
% hObject    handle to zoomButton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
zoom on;


% --- Executes on button press in panButton.
function panButton_Callback(hObject, eventdata, handles)
% hObject    handle to panButton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
pan on;


% --- Executes on button press in importVertsButton.
function importVertsButton_Callback(hObject, eventdata, handles)
% hObject    handle to importVertsButton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

    error=0;
    try
        verts=evalin('base',get(handles.importVertsInput,'String'));
    catch
       msgbox('Attempted import of vertices failed.  Data must be a two column matrix of points.','NCS Toolbox Error','error')
       return;
    end
    
    contents = get(handles.vertList,'String');
    for ctr=1:size(verts,1)
        x=verts(ctr,1);
        y=verts(ctr,2);
        if x>=y
            updateFigure(handles,[x y],'refined_add');
        else
            error=1;
        end
    end
    
    if error
        msgbox('One or more specified vertices were not added since they lie outside the uncertain region','NCS Toolbox Error','error')
        return;
    end
    
    
function importVertsInput_Callback(hObject, eventdata, handles)
% hObject    handle to importVertsInput (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of importVertsInput as text
%        str2double(get(hObject,'String')) returns contents of importVertsInput as a double


% --- Executes during object creation, after setting all properties.
function importVertsInput_CreateFcn(hObject, eventdata, handles)
% hObject    handle to importVertsInput (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in genPolyButton.
function genPolyButton_Callback(hObject, eventdata, handles)
% hObject    handle to genPolyButton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

try
    delta = str2double(get(handles.thresholdBox,'String'));
catch
    msgbox('Threshold must be a number greater than zero and less than or equal to one.','NCS Toolbox Error','error')
    return;
end

if ~isnumeric(delta) || isnan(delta)
    msgbox('Threshold must be a number greater than zero and less than or equal to one.','NCS Toolbox Error','error')
    return;
end

if(delta< 0)
    msgbox('Threshold must be a non-negative number.','NCS Toolbox Error','error')
    return;
end

temp=get(handles.axes1,'UserData');         %retrieve currently plotted data
timingDist=temp{7};
if isempty(timingDist)
   msgbox('Must plot measurement data before auto identify feature can be used.','NCS Toolbox Error','error')
   return;
end 

set(hObject,'Enable','off');
drawnow;

%determine refined region
measRes=str2double(get(handles.measResBox,'String'));
[pt timingDist tPoly Q]=genUncertainPolyFromMeasurementData(timingDist,delta,measRes);

if isempty(tPoly)
   msgbox('No measurement points meet specified threshold criteria.','NCS Toolbox Error','error')
   set(hObject,'Enable','on');
   return;
end 
updateFigure(handles,tPoly,'refined_new');
Q=round(Q*100)/100;
set(handles.Q,'String',['(includes ' num2str(Q) '% of measurement data)']);
set(handles.Q,'Visible','on');

set(hObject,'Enable','on');



function hmin_Callback(hObject, eventdata, handles)
% hObject    handle to hmin (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of hmin as text
%        str2double(get(hObject,'String')) returns contents of hmin as a double
%draw unRefined region
    
    hmin = str2double(get(handles.hmin,'String'));
    hmax = str2double(get(handles.hmax,'String'));
    taumin = str2double(get(handles.taumin,'String'));
    taumax = str2double(get(handles.taumax,'String'));

    error=checkTimingBounds([hmin hmax taumin taumax]);

    if error
       set(hObject,'String','');
       msgbox(error,'NCS Toolbox Error','error');
    end

    if ~error && ~isnan(hmin) && ~isnan(hmax) && ~isnan(taumin) && ~isnan(taumax)   
        verts=genTimingUncertaintyRegion(hmin,hmax,taumin,taumax);
        updateFigure(handles,verts,'unRefined');
        handles.uncertainBounds=verts;
        guidata(handles.timingGui, handles);
    end
    
% --- Executes during object creation, after setting all properties.
function hmin_CreateFcn(hObject, eventdata, handles)
% hObject    handle to hmin (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function hmax_Callback(hObject, eventdata, handles)
% hObject    handle to hmax (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of hmax as text
%        str2double(get(hObject,'String')) returns contents of hmax as a double

    
    
    hmin = str2double(get(handles.hmin,'String'));
    hmax = str2double(get(handles.hmax,'String'));
    taumin = str2double(get(handles.taumin,'String'));
    taumax = str2double(get(handles.taumax,'String'));

    error=checkTimingBounds([hmin hmax taumin taumax]);

    if error
       set(hObject,'String','');
       msgbox(error,'NCS Toolbox Error','error');
    end

    if ~error && ~isnan(hmin) && ~isnan(hmax) && ~isnan(taumin) && ~isnan(taumax)   
        verts=genTimingUncertaintyRegion(hmin,hmax,taumin,taumax);
        updateFigure(handles,verts,'unRefined');
        handles.uncertainBounds=verts;
        guidata(handles.timingGui, handles);
    end

% --- Executes during object creation, after setting all properties.
function hmax_CreateFcn(hObject, eventdata, handles)
% hObject    handle to hmax (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function taumin_Callback(hObject, eventdata, handles)
% hObject    handle to taumin (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of taumin as text
%        str2double(get(hObject,'String')) returns contents of taumin as a double
    
    hmin = str2double(get(handles.hmin,'String'));
    hmax = str2double(get(handles.hmax,'String'));
    taumin = str2double(get(handles.taumin,'String'));
    taumax = str2double(get(handles.taumax,'String'));

    error=checkTimingBounds([hmin hmax taumin taumax]);

    if error
       set(hObject,'String','');
       msgbox(error,'NCS Toolbox Error','error');
    end

    if ~error && ~isnan(hmin) && ~isnan(hmax) && ~isnan(taumin) && ~isnan(taumax)   
        verts=genTimingUncertaintyRegion(hmin,hmax,taumin,taumax);
        updateFigure(handles,verts,'unRefined');
        handles.uncertainBounds=verts;
        guidata(handles.timingGui, handles);
    end
    
% --- Executes during object creation, after setting all properties.
function taumin_CreateFcn(hObject, eventdata, handles)
% hObject    handle to taumin (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function taumax_Callback(hObject, eventdata, handles)
% hObject    handle to taumax (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of taumax as text
%        str2double(get(hObject,'String')) returns contents of taumax as a double
   
    
    hmin = str2double(get(handles.hmin,'String'));
    hmax = str2double(get(handles.hmax,'String'));
    taumin = str2double(get(handles.taumin,'String'));
    taumax = str2double(get(handles.taumax,'String'));

    error=checkTimingBounds([hmin hmax taumin taumax]);

    if error
       set(hObject,'String','');
       msgbox(error,'NCS Toolbox Error','error');
    end

    if ~error && ~isnan(hmin) && ~isnan(hmax) && ~isnan(taumin) && ~isnan(taumax)   
        verts=genTimingUncertaintyRegion(hmin,hmax,taumin,taumax);
        updateFigure(handles,verts,'unRefined');
        handles.uncertainBounds=verts;
        guidata(handles.timingGui, handles);
    end
    

% --- Executes during object creation, after setting all properties.
function taumax_CreateFcn(hObject, eventdata, handles)
% hObject    handle to taumax (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function measResBox_Callback(hObject, eventdata, handles)
% hObject    handle to measResBox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of measResBox as text
%        str2double(get(hObject,'String')) returns contents of measResBox as a double


% --- Executes during object creation, after setting all properties.
function measResBox_CreateFcn(hObject, eventdata, handles)
% hObject    handle to measResBox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in clearButton.
function clearButton_Callback(hObject, eventdata, handles)
% hObject    handle to clearButton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

    set(hObject,'Enable','off');
    updateFigure(handles,[],'plot_data');
    set(handles.measurementDataBox,'UserData',[]);
    set(handles.probabilityLabel,'Visible','off');
    set(handles.thresholdBox,'Enable','off');
    set(handles.genPolyButton,'Enable','off');
    colorbar('delete');


% --- Executes on button press in minMaxFitButton.
function minMaxFitButton_Callback(hObject, eventdata, handles)
% hObject    handle to minMaxFitButton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

hmin    = str2double(get(handles.hmin,'String'));
hmax    = str2double(get(handles.hmax,'String'));
taumin 	= str2double(get(handles.taumin,'String'));
taumax  = str2double(get(handles.taumax,'String'));

if ~isnan(hmin) && ~isnan(hmax) && ~isnan(taumin) && ~isnan(taumax)
    if hmin==hmax %the bound is a vertical line
        if(hmax < taumax)
            taumax=hmax;
        end       
    end
    if taumin==taumax %the bound is a horiontal line
        if(hmin < taumax)
            hmin=taumax;
        end       
    end    
    tPoly = genTimingUncertaintyRegion(hmin,hmax,taumin,taumax);
    updateFigure(handles,tPoly,'refined_new');
else
    msgbox('Bounds must be decimal values.','NCS Toolbox Error','error')
end




% --- Executes during object creation, after setting all properties.
function probabilityLabel_CreateFcn(hObject, eventdata, handles)
% hObject    handle to probabilityLabel (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called


% --- Executes during object creation, after setting all properties.
function importVertsButton_CreateFcn(hObject, eventdata, handles)
% hObject    handle to importVertsButton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called
